@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.55 2003/12/16 19:47:43 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.62 2003/12/20 04:20:50 fuyu Exp $
  */
 
 #include "config.h"
@@ -59,6 +59,11 @@ typedef struct _PopupContext PopupContext;
 static void link_popup_menu_callback(gpointer data, guint action,
 				     GtkWidget *widget);
 static void link_popup_menu_destructed_cb(gpointer data);
+
+static void response_popup_menu_callback(gpointer data, guint action,
+				     GtkWidget *widget);
+static void response_popup_menu_destructed_cb(gpointer data);
+
 static gboolean link_mouse_over_cb(BBSThreadView *view, GdkEventMotion *event,
 				   OchushaBBSThread *thread, const gchar *link,
 				   OchushaApplication *application);
@@ -74,6 +79,8 @@ static gboolean link_mouse_release_cb(BBSThreadView *view,
 				      OchushaBBSThread *thread,
 				      const gchar *link,
 				      OchushaApplication *application);
+static void scroll_view_cb(GtkWidget *view, GtkScrollType scroll,
+			   OchushaApplication *application);
 static void write_response_cb(BBSThreadView *view,
 			      OchushaApplication *application);
 static void interactive_search_cb(BBSThreadView *view,
@@ -90,7 +97,8 @@ static void show_popup(OchushaApplication *application,
 static gboolean popup_delay_timeout(PopupContext *popup_context);
 static void show_popup_real(PopupContext *popup_context);
 static void hide_popup(PopupContext *popup_context);
-
+static void insert_bookmark(BBSThreadView *view, OchushaBBSThread *thread,
+			    int res_num, OchushaApplication *application);
 static void render_bbs_thread(WorkerThread *employee, gpointer args);
 static gboolean start_thread_rendering(OchushaBBSThread *thread,
 				       const gchar *title,
@@ -108,7 +116,6 @@ static void start_element_cb(void *context, const char *name,
 static void end_element_cb(void *context, const char *name);
 static void characters_cb(void *context, const char *ch, int len);
 
-static pthread_mutex_t thread_info_lock;
 static pthread_mutex_t bbs_thread_ui_lock;
 
 /* ơС˥ᥤȤʤɤɽ뤿context ID */
@@ -124,6 +131,8 @@ static guint message_id;
 guint thread_netstat_id;
 
 
+#if 0	/* Ϥפäݤ */
+static pthread_mutex_t thread_info_lock;
 #define THREAD_INFO_LOCK				\
   if (pthread_mutex_lock(&thread_info_lock) != 0)	\
     {							\
@@ -137,6 +146,7 @@ guint thread_netstat_id;
       fprintf(stderr, "Couldn't unlock a mutex.\n");	\
       abort();						\
     }
+#endif
 
 
 #define BBS_THREAD_UI_LOCK				\
@@ -159,8 +169,11 @@ typedef struct _RendererContext
   ElementHandler handler;
   BBSThreadView *view;
   OchushaBBSThread *thread;
+  BBSThreadGUIInfo *info;
   OchushaApplication *application;
   gint last_read_number;
+  gint last_rendered_response;
+  gboolean title_shown;
 
   gchar *link;
   gchar *link_text_buffer;
@@ -226,7 +239,13 @@ enum {
 };
 
 
+enum {
+  RESPONSE_POPUP_MENU_INSERT_BOOKMARK = 1,
+};
+
+
 static GtkItemFactory *link_popup_menu_item_factory = NULL;
+static GtkItemFactory *response_popup_menu_item_factory = NULL;
 static GQuark bbs_thread_gui_info_id;
 static GQuark popup_context_id;
 static GQuark popup_parent_context_id;
@@ -235,6 +254,14 @@ static GQuark popup_parent_context_id;
 void
 prepare_thread_ui_initialization(OchushaApplication *application)
 {
+#ifdef THREAD_INFO_LOCK
+  if (pthread_mutex_init(&thread_info_lock, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a mutex.\n");
+      abort();
+    }
+#endif
+
   if (pthread_mutex_init(&bbs_thread_ui_lock, NULL) != 0)
     {
       fprintf(stderr, "Couldn't init a mutex.\n");
@@ -520,12 +547,33 @@ initialize_thread_ui(OchushaApplication *application)
       },
     };
   gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
+
+  GtkItemFactoryEntry response_menu_items[] =
+    {
+      {
+	_("/Insert Bookmark"),				/* menu path */
+	NULL,						/* accelerator */
+	response_popup_menu_callback,			/* callback_func */
+	RESPONSE_POPUP_MENU_INSERT_BOOKMARK,		/* act */
+	NULL						/* type */
+      },
+    };
+  gint response_nmenu_items
+    = sizeof(response_menu_items) / sizeof(response_menu_items[0]);
+
   link_popup_menu_item_factory = gtk_item_factory_new(GTK_TYPE_MENU,
 						      "<ThreadLinkPopup>",
 						      NULL);
   gtk_item_factory_create_items(link_popup_menu_item_factory,
 				nmenu_items, menu_items, application);
 
+  response_popup_menu_item_factory = gtk_item_factory_new(GTK_TYPE_MENU,
+							  "<ResponsePopup>",
+							  NULL);
+  gtk_item_factory_create_items(response_popup_menu_item_factory,
+				response_nmenu_items, response_menu_items,
+				application);
+
   /* ̤εǽ˥塼Ĥ֤ */
   dummy_widget = gtk_item_factory_get_widget_by_action(
 					link_popup_menu_item_factory,
@@ -622,6 +670,16 @@ bbs_thread_info_free(BBSThreadGUIInfo *info)
 BBSThreadGUIInfo *
 ensure_bbs_thread_info(OchushaBBSThread *thread)
 {
+#ifndef THREAD_INFO_LOCK
+  BBSThreadGUIInfo *info = g_object_get_qdata(G_OBJECT(thread),
+					      bbs_thread_gui_info_id);
+  if (info == NULL)
+    {
+      info = G_NEW0(BBSThreadGUIInfo, 1);
+      g_object_set_qdata_full(G_OBJECT(thread), bbs_thread_gui_info_id,
+			      info, (GDestroyNotify)bbs_thread_info_free);
+    }
+#else
   BBSThreadGUIInfo *info = g_object_get_qdata(G_OBJECT(thread),
 					      bbs_thread_gui_info_id);
   if (info != NULL)
@@ -638,6 +696,7 @@ ensure_bbs_thread_info(OchushaBBSThread *thread)
       }
   }
   THREAD_INFO_UNLOCK;
+#endif
 
   return info;
 }
@@ -756,6 +815,8 @@ open_bbs_thread(OchushaApplication *application, OchushaBBSThread *thread,
 		     G_CALLBACK(write_response_cb), application);
     g_signal_connect(G_OBJECT(thread_view), "interactive_search",
 		     G_CALLBACK(interactive_search_cb), application);
+    g_signal_connect(G_OBJECT(thread_view), "scroll_view",
+		     G_CALLBACK(scroll_view_cb), application);
 
 #if DEBUG_ASYNC_BUFFER_MOST
     fprintf(stderr, "open_bbs_thread: ref AsyncBuffer(%p)\n", buffer);
@@ -818,6 +879,41 @@ link_popup_menu_destructed_cb(gpointer data)
 }
 
 
+typedef struct _ResponsePopupMenuData
+{
+  int res_num;
+  BBSThreadView *view;
+  OchushaBBSThread *thread;
+} ResponsePopupMenuData;
+
+
+static void
+response_popup_menu_callback(gpointer data, guint action, GtkWidget *widget)
+{
+  ResponsePopupMenuData *menu_data
+    = gtk_item_factory_popup_data(response_popup_menu_item_factory);
+
+  switch (action)
+    {
+    case RESPONSE_POPUP_MENU_INSERT_BOOKMARK:
+      insert_bookmark(menu_data->view, menu_data->thread,
+		      menu_data->res_num, (OchushaApplication*)data);
+      break;
+    }
+}
+
+
+static void
+response_popup_menu_destructed_cb(gpointer data)
+{
+#if DEBUG_GUI_MOST
+  fprintf(stderr, "response_popup_menu_destructed_cb: url=\"%s\"\n",
+	  (char *)data);
+#endif
+  G_FREE(data);
+}
+
+
 static gboolean
 link_mouse_over_cb(BBSThreadView *view, GdkEventMotion *event,
 		   OchushaBBSThread *thread, const gchar *link,
@@ -831,11 +927,11 @@ link_mouse_over_cb(BBSThreadView *view, GdkEventMotion *event,
     {
       gtk_statusbar_remove(statusbar, link_field_id, message_id);
       stack_depth--;
+      message_id = 0;
     }
 
-#if DEBUG_POPUP
-  fprintf(stderr, "link_mouse_over_cb: show_popup()\n");
-#endif
+  if (strncmp(link, "<RES>", 5) == 0)
+    return FALSE;
 
   if (application->enable_popup_response)
     show_popup(application, thread, link, event->x_root, event->y_root, view);
@@ -933,6 +1029,7 @@ ensure_popup_context(BBSThreadView *parent, OchushaBBSThread *thread,
   popup_context = G_NEW0(PopupContext, 1);
 
   renderer_context = &popup_context->renderer_context;
+  renderer_context->info = ensure_bbs_thread_info(thread);
 
   /* renderer_context->handler.converter = 0; */
   renderer_context->handler.startElement = start_element_cb;
@@ -1092,8 +1189,8 @@ link_mouse_out_cb(BBSThreadView *view, GdkEventAny *event,
     {
       gtk_statusbar_remove(statusbar, link_field_id, message_id);
       message_id = 0;
+      stack_depth--;
     }
-  stack_depth--;
 
 #if DEBUG_POPUP
   fprintf(stderr, "link_mouse_out_cb: hide_popup(%p)\n", popup_context);
@@ -1112,7 +1209,30 @@ link_mouse_press_cb(BBSThreadView *view, GdkEventButton *event,
   if (link == NULL)
     return FALSE;
 
-  if (event->button == 3)
+  if (strncmp(link, "<RES>", 5) == 0)
+    {
+      int res_num;
+      if (!application->stick_bookmark)
+	return FALSE;
+
+      res_num = 0;
+      if (event->button == 3 && sscanf(link + 5, "%d", &res_num) == 1)
+	{
+	  ResponsePopupMenuData *popup_data
+	    = G_NEW0(ResponsePopupMenuData, 1);
+	  popup_data->res_num = res_num;
+	  popup_data->view = view;
+	  popup_data->thread = thread;
+
+	  gtk_item_factory_popup_with_data(response_popup_menu_item_factory,
+					   popup_data,
+					   response_popup_menu_destructed_cb,
+					   event->x_root, event->y_root,
+					   event->button,
+					   gtk_get_current_event_time());
+	}
+    }
+  else if (event->button == 3)
     gtk_item_factory_popup_with_data(link_popup_menu_item_factory,
 				     G_STRDUP(link),
 				     link_popup_menu_destructed_cb,
@@ -1135,6 +1255,14 @@ link_mouse_release_cb(BBSThreadView *view, GdkEventButton *event,
 }
 
 
+static void
+scroll_view_cb(GtkWidget *view, GtkScrollType scroll,
+	       OchushaApplication *application)
+{
+  g_signal_emit_by_name(G_OBJECT(view->parent), "scroll_child", scroll, FALSE);
+}
+
+
 static void
 write_response_cb(BBSThreadView *view, OchushaApplication *application)
 {
@@ -1205,12 +1333,132 @@ save_initial_thread_view_position(BBSThreadView *view, BBSThreadGUIInfo *info,
   info->next_mark = mark;
   args->view = view;
   args->info = info;
-  info->scroller_id = g_idle_add_full(G_PRIORITY_DEFAULT_IDLE,
+  info->scroller_id = g_idle_add_full(G_PRIORITY_HIGH_IDLE + 13,
 				      (GSourceFunc)scroll_to_mark_on_idle,
 				      args, (GDestroyNotify)g_free);
 }
 
 
+static void
+bookmark_clicked_cb(GtkWidget *widget, BBSThreadView *view)
+{
+  OchushaApplication *application = g_object_get_data(G_OBJECT(widget),
+						      "application");
+  OchushaBBSThread *thread = g_object_get_data(G_OBJECT(widget),
+					       "thread");
+  BBSThreadGUIInfo *info;
+
+  g_return_if_fail(application != NULL && OCHUSHA_IS_BBS_THREAD(thread));
+
+#if 0	/* ̤ɤġġ */
+  if (!application->stick_bookmark)
+    return;
+#endif
+
+  info = ensure_bbs_thread_info(thread);
+
+  if (info->bookmark_response_number == thread->number_of_responses_read)
+    return;
+
+  insert_bookmark(view, thread, thread->number_of_responses_read, application);
+}
+
+
+static void
+bookmark_destroy_cb(GtkWidget *widget, BBSThreadGUIInfo *info)
+{
+}
+
+
+static void
+insert_bookmark(BBSThreadView *view, OchushaBBSThread *thread, int res_num,
+		OchushaApplication *application)
+{
+  BBSThreadGUIInfo *info;
+  GtkWidget *bookmark;
+  guint *offsets;
+  guint offsets_length;
+  int i;
+  gchar *bookmark_text;
+
+  info = ensure_bbs_thread_info(thread);
+  offsets = info->offsets;
+  offsets_length = info->offsets_length;
+
+  g_return_if_fail(res_num > 0 && res_num < offsets_length);
+
+  if (info->bookmark != NULL && info->bookmark_response_number == res_num)
+    return;
+
+  bookmark_text = application->bookmark_text;
+  
+  if (bookmark_text == NULL || *bookmark_text == '\0')
+    {
+      if (application->bookmark_text != NULL)
+	G_FREE(application->bookmark_text);
+      bookmark_text = G_STRDUP(_("==== My bookmark: I have read above. ===="));
+      application->bookmark_text = bookmark_text;
+    }
+
+  bookmark = gtk_button_new_with_label(bookmark_text);
+  GTK_WIDGET_UNSET_FLAGS(bookmark, GTK_CAN_FOCUS);
+
+  gtk_widget_set_name(bookmark, "mybookmark");
+
+  g_object_set_data(G_OBJECT(bookmark), "application", application);
+  g_object_set_data(G_OBJECT(bookmark), "thread", thread);
+
+  if (info->bookmark != NULL)
+    {
+      bbs_thread_view_remove_widget(view, info->bookmark);
+      info->bookmark = NULL;
+
+      for (i = info->bookmark_response_number + 1; i < offsets_length; i++)
+	{
+	  if (offsets[i] == 0)
+	    break;
+	  offsets[i]--;
+	}
+    }
+
+  g_signal_connect(G_OBJECT(bookmark), "clicked",
+		   G_CALLBACK(bookmark_clicked_cb), view);
+  g_signal_connect(G_OBJECT(bookmark), "destroy",
+		   G_CALLBACK(bookmark_destroy_cb), info);
+
+  if (info->last_read != NULL)
+    bbs_thread_view_delete_mark(view, info->last_read);
+
+  if (offsets[res_num + 1] == 0)
+    {
+      if (offsets[res_num] != 0)
+	info->last_read
+	  = bbs_thread_view_create_mark_at_offset(view, offsets[res_num]);
+      else
+	info->last_read = bbs_thread_view_create_mark(view);
+      bbs_thread_view_append_widget(view, bookmark);
+    }
+  else
+    {
+      info->last_read
+	= bbs_thread_view_create_mark_at_offset(view, offsets[res_num]);
+      bbs_thread_view_insert_widget_at_offset(view, bookmark,
+					      offsets[res_num + 1]);
+      for (i = res_num + 1; i < offsets_length; i++)
+	{
+	  if (offsets[i] == 0)
+	    break;
+	  offsets[i]++;
+	}
+    }
+
+  info->bookmark_response_number = res_num;
+  info->bookmark = bookmark;
+
+  update_threadlist_entry(application, thread);
+}
+
+
 static void
 render_bbs_thread(WorkerThread *employee, gpointer args)
 {
@@ -1243,8 +1491,11 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
       },					/* handler */
       job_args->view,				/* view */
       thread,					/* thread */
+      info,					/* info */
       application,				/* application */
       thread->number_of_responses_read,		/* last_read_number */
+      job_args->start,				/* last_rendered_response */
+      job_args->start > 0,			/* title_shown */
 
       NULL,					/* link */
       default_text_buffer,			/* link_text_buffer */
@@ -1381,10 +1632,6 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
       info->offsets = G_NEW0(guint, MAX_RESPONSE);
       info->offsets_length = MAX_RESPONSE;
     }
-  else 
-    {
-      memset(info->offsets, 0, info->offsets_length * sizeof(guint));
-    }
 
   if (context.handler.converter == (iconv_t)-1)
     {
@@ -1415,30 +1662,20 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
 
   if (job_args->refresh)
     {
-      int index = job_args->start - 2;
-      if (info->last_read != NULL)
-	bbs_thread_view_delete_mark(job_args->view, info->last_read);
-
-      /* ɥ쥹ΤˤϺǸδɥ쥹ľˤ˰դ롣*/
-      if (index > 0
-	  && info->offsets_length > index && info->offsets[index] != 0)
-	{
-	  info->last_read
-	    = bbs_thread_view_create_mark_at_offset(job_args->view,
-						    info->offsets[index]);
-	}
-      else
-	info->last_read = bbs_thread_view_create_mark(job_args->view);
-
-      save_initial_thread_view_position(job_args->view, info, info->last_read);
-#if DEBUG_GUI
-      fprintf(stderr, "create mark at the tail response.\n");
-#endif
       snprintf(default_text_buffer, DEFAULT_BUFFER_SIZE,
 	       _("Updating thread %s@%s board"),
 	       thread->title, thread->board->name);
 
       gdk_threads_enter();
+      if (!application->stick_bookmark)
+	{
+	  insert_bookmark(job_args->view, thread, job_args->start,
+			  application);
+
+	  save_initial_thread_view_position(job_args->view, info,
+					    info->last_read);
+	}
+
       if (info->message_id != 0)
 	gtk_statusbar_remove(application->statusbar, thread_netstat_id,
 			     info->message_id);
@@ -1475,27 +1712,13 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
 
   update_threadlist_entry(job_args->application, thread);
 
-#if DEBUG_WIDGET_MOST
-  fprintf(stderr, "render_bbs_thread<done>: view=%p, view->ref_count=%d\n",
-	  job_args->view, G_OBJECT(job_args->view)->ref_count);
-#endif
-
   if (job_args->tab_label != NULL)
     g_object_unref(G_OBJECT(job_args->tab_label));
 
-#if DEBUG_WIDGET
-  fprintf(stderr, "render_bbs_thread: unref BBSThreadView(%p) ref_count=%d\n",
-	  job_args->view, G_OBJECT(job_args->view)->ref_count);
-#endif
   g_object_unref(G_OBJECT(job_args->view));
 
   iconv_close(context.handler.converter);
 
-#if DEBUG_ASYNC_BUFFER_MOST
-  fprintf(stderr, "render_bbs_thread: unref AsyncBuffer(%p)\n", buffer);
-  fprintf(stderr, "* before unref: buffer->ref_count=%d\n",
-	  G_OBJECT(buffer)->ref_count);
-#endif
   g_object_unref(G_OBJECT(buffer));
 
   gdk_threads_enter();
@@ -1538,10 +1761,6 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
   if (context.link_text_buffer != default_text_buffer)
     G_FREE(context.link_text_buffer);
   G_FREE(args);
-
-#if DEBUG_GUI_MOST
-  fprintf(stderr, "render_bbs_thread() done.\n");
-#endif
 }
 
 
@@ -1551,12 +1770,17 @@ start_thread_rendering(OchushaBBSThread *thread, const gchar *title,
 {
   RendererContext *context = (RendererContext *)user_data;
   BBSThreadView *view = context->view;
-  BBSThreadGUIInfo *info = ensure_bbs_thread_info(context->thread);
+  BBSThreadGUIInfo *info = context->info;
 
 #if DEBUG_GUI_MOST
   fprintf(stderr, "start_rendering\n");
 #endif
 
+  if (context->title_shown)
+    return TRUE;
+
+  context->title_shown = TRUE;
+
   gdk_threads_enter();
 
   if (context->last_read_number == 0 && info->res_num == 0)
@@ -1596,8 +1820,8 @@ render_response(OchushaBBSThread *thread, gint number,
 {
   RendererContext *context = (RendererContext *)user_data;
   BBSThreadView *view = context->view;
-  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
-  char buffer[13];
+  BBSThreadGUIInfo *info = context->info;
+  char buffer[18];
   gboolean a_bone;
 
 #if DEBUG_GUI_MOST
@@ -1605,6 +1829,11 @@ render_response(OchushaBBSThread *thread, gint number,
 	  number, context->last_read_number);
 #endif
 
+  if (context->last_rendered_response >= number)
+    return TRUE;
+
+  context->last_rendered_response = number;
+
   if (context->a_bone_by_name != NULL
       && oniguruma_regex_match(context->a_bone_by_name, response->name))
     a_bone = TRUE;
@@ -1637,28 +1866,28 @@ render_response(OchushaBBSThread *thread, gint number,
       save_initial_thread_view_position(view, info, mark);
     }
 
-  if (number == context->last_read_number)
-    {
-      info->last_read = bbs_thread_view_create_mark(view);
-      if (info->res_num == 0)
-	save_initial_thread_view_position(view, info, info->last_read);
-    }
-
   if (thread->number_of_responses_read < number)
-    {
-      thread->number_of_responses_read = number;
-      update_threadlist_entry(context->application, thread);
-    }
+    thread->number_of_responses_read = number;
+
+  info->num_shown = number;
+  update_threadlist_entry(context->application, thread);
 
   if (a_bone && context->transparent_a_bone)
-    return TRUE;
+    {
+      gdk_threads_enter();
+      goto after_response;
+    }
 
-  snprintf(buffer, 13, "\n%d", number);
+  snprintf(buffer, 18, "<RES>%d", number);
 
   gdk_threads_enter();
 
   bbs_thread_view_push_tag(view, context->response_header_tag);
-  bbs_thread_view_append_text(view, buffer, -1);
+  bbs_thread_view_append_text(view, "\n", 1);
+  if (context->last_read_number >= 0)
+    bbs_thread_view_append_text_as_link(view, buffer + 5, -1, buffer);
+  else
+    bbs_thread_view_append_text(view, buffer + 5, -1);
   bbs_thread_view_append_text(view, LARGE_COLON, 3);
 
   if (!a_bone)
@@ -1781,6 +2010,26 @@ render_response(OchushaBBSThread *thread, gint number,
 
   bbs_thread_view_append_text(view, "\n", 1);
 
+ after_response:
+  if (context->application->stick_bookmark)
+    {
+      if (number == info->bookmark_response_number)
+	{
+	  insert_bookmark(view, context->thread, number, context->application);
+	  if (info->res_num == 0)
+	    save_initial_thread_view_position(view, info, info->last_read);
+	}
+    }
+  else
+    {
+      if (number == context->last_read_number)
+	{
+	  insert_bookmark(view, context->thread, number, context->application);
+	  if (info->res_num == 0)
+	    save_initial_thread_view_position(view, info, info->last_read);
+	}
+    }
+
   gdk_threads_leave();
 
   return TRUE;
@@ -1793,13 +2042,18 @@ render_broken_response(OchushaBBSThread *thread, gint number,
 {
   RendererContext *context = (RendererContext *)user_data;
   BBSThreadView *view = context->view;
-  BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
+  BBSThreadGUIInfo *info = context->info;
   char buffer[13];
 #if DEBUG_GUI_MOST
   fprintf(stderr, "rendering response #%d, last_read_number=%d\n",
 	  number, context->last_read_number);
 #endif
 
+  if (context->last_rendered_response >= number)
+    return TRUE;
+
+  context->last_rendered_response = number;
+
   if (context->last_read_number >= 0)
     {
       if (number >= info->offsets_length)
@@ -1819,18 +2073,11 @@ render_broken_response(OchushaBBSThread *thread, gint number,
       save_initial_thread_view_position(view, info, mark);
     }
   
-  if (number == context->last_read_number)
-    {
-      info->last_read = bbs_thread_view_create_mark(view);
-      if (info->res_num == 0)
-	save_initial_thread_view_position(view, info, info->last_read);
-    }
-
   if (thread->number_of_responses_read < number)
-    {
-      thread->number_of_responses_read = number;
-      update_threadlist_entry(context->application, thread);
-    }
+    thread->number_of_responses_read = number;
+
+  info->num_shown = number;
+  update_threadlist_entry(context->application, thread);
 
   snprintf(buffer, 13, "\n%d", number);
 
@@ -1845,6 +2092,25 @@ render_broken_response(OchushaBBSThread *thread, gint number,
   bbs_thread_view_append_text(view, _("[broken here]\n"), -1);
   bbs_thread_view_pop_tags(view, context->response_paragraph_tag);
 
+  if (context->application->stick_bookmark)
+    {
+      if (number == info->bookmark_response_number)
+	{
+	  insert_bookmark(view, context->thread, number, context->application);
+	  if (info->res_num == 0)
+	    save_initial_thread_view_position(view, info, info->last_read);
+	}
+    }
+  else
+    {
+      if (number == context->last_read_number)
+	{
+	  insert_bookmark(view, context->thread, number, context->application);
+	  if (info->res_num == 0)
+	    save_initial_thread_view_position(view, info, info->last_read);
+	}
+    }
+
   gdk_threads_leave();
 
   return TRUE;
@@ -1857,23 +2123,9 @@ end_thread_rendering(OchushaBBSThread *thread, gboolean finished,
 {
   RendererContext *context = (RendererContext *)user_data;
   BBSThreadView *view = context->view;
-  BBSThreadGUIInfo *info = ensure_bbs_thread_info(context->thread);
+  BBSThreadGUIInfo *info = context->info;
 
-  if (!finished)
-    {
-      /*
-       * MEMO: å夬äƤ(ܡ󤵤줿쥹)
-       *       å夵Ƥ̤ͤ꾯ʤäʤɡ
-       *       DATեΤμľȤ롣
-       */
-#if DEBUG_GUI_MOST
-      fprintf(stderr, "end_thread_rendering: clear current thread view\n");
-#endif
-      gdk_threads_enter();
-      bbs_thread_view_clear(view);
-      gdk_threads_leave();
-    }
-  else
+  if (finished)
     {
       if (info->last_read == NULL)
 	{
@@ -1888,6 +2140,40 @@ end_thread_rendering(OchushaBBSThread *thread, gboolean finished,
 	}
       info->res_num = 0;
     }
+  else
+    {
+      gchar message[DEFAULT_BUFFER_SIZE];
+#if DEBUG_GUI_MOST
+      fprintf(stderr, "end_thread_rendering: a bone happen!?\n");
+#endif
+      /*
+       * MEMO: å夬äƤ(ܡ󤵤줿쥹)
+       *       å夵Ƥ̤ͤ꾯ʤäʤɡ
+       *       DATեΤμľȤ뤬
+       *       DATեβϥ롼ϡꤷ쥹ְʹߤ
+       *       ƤΤcallbackؿƤ֤Τǡι
+       *       å夬äƤΤȽˡ
+       *       äƤޤȡ쥹ɽʤȤˤʤ롣
+       *       ʤΤǡΤˤȤʤɽä
+       *       ʬϡܡ󤵤ƤƤ⸵Υ쥹ɽ뤳Ȥ
+       *       롣
+       */
+      snprintf(message, DEFAULT_BUFFER_SIZE,
+	       _("A Bone occurs in thread %s@%s board"),
+	       thread->title, thread->board->name);
+
+      gdk_threads_enter();
+      if (info->message_id != 0)
+	gtk_statusbar_remove(context->application->statusbar,
+			     thread_netstat_id, info->message_id);
+      info->message_id = gtk_statusbar_push(context->application->statusbar,
+					    thread_netstat_id, message);
+      gdk_threads_leave();
+      thread->number_of_responses_read = 1;
+      /* 0ˤȡɤ߹ߤߤˡå夬ĤäƤΤ̤
+       * ˤʤ륿ߥ󥰤Τǡ
+       */
+    }
 
 #if DEBUG_GUI_MOST
   fprintf(stderr, "end_rendering\n");
@@ -2483,6 +2769,7 @@ render_popup_response(PopupContext *popup_context,
       return;
     }
 
+  popup_context->renderer_context.last_rendered_response = 0;
   gdk_threads_leave();
   ochusha_bbs_thread_parse_responses(thread, buffer,
 				     response_from - 1, num_responses,
@@ -2902,7 +3189,7 @@ go_to_the_last_response(GtkWidget *widget, OchushaBBSThread *thread)
 
 
 void
-jump_to_new_comer_response(GtkWidget *widget, OchushaBBSThread *thread)
+jump_to_bookmark(GtkWidget *widget, OchushaBBSThread *thread)
 {
   BBSThreadView *view;
   BBSThreadGUIInfo *info;
