@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.26 2003/12/12 21:43:52 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.29 2003/12/18 15:56:34 fuyu Exp $
  */
 
 #include "config.h"
@@ -124,6 +124,7 @@ enum {
   VIEW_ENTER_NOTIFY_SIGNAL,
   VIEW_LEAVE_NOTIFY_SIGNAL,
   SCROLL_TO_SIGNAL,
+  SCROLL_VIEW_SIGNAL,
   INTERACTIVE_SEARCH_SIGNAL,
   WRITE_RESPONSE_SIGNAL,
   CLOSE_THREAD_SIGNAL,
@@ -139,7 +140,7 @@ enum {
 
 static GtkTextViewClass *parent_class = NULL;
 static gint bbs_thread_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkTextTagTable *default_tag_table;
 
 static GtkTextTag *link_tags[2];
@@ -354,6 +355,15 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 		 ochusha_marshal_VOID__INT,
 		 G_TYPE_NONE, 1,
 		 G_TYPE_INT);
+  bbs_thread_view_signals[SCROLL_VIEW_SIGNAL] =
+    g_signal_new("scroll_view",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+		 G_STRUCT_OFFSET(BBSThreadViewClass, scroll_view),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__ENUM,
+		 G_TYPE_NONE, 1,
+		 GTK_TYPE_SCROLL_TYPE);
 
   bbs_thread_view_signals[INTERACTIVE_SEARCH_SIGNAL] =
     g_signal_new("interactive_search",
@@ -397,15 +407,17 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
 			       G_TYPE_INT, SCROLL_TO_BUFFER_END);
 
   gtk_binding_entry_add_signal(binding_set, GDK_space, 0,
-			       "move_cursor", 3,
-			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
-			       G_TYPE_INT, 1,
-			       G_TYPE_BOOLEAN, FALSE);
+			       "scroll_view", 1,
+			       GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_FORWARD);
+  gtk_binding_entry_add_signal(binding_set, GDK_Page_Down, 0,
+			       "scroll_view", 1,
+			       GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_FORWARD);
   gtk_binding_entry_add_signal(binding_set, GDK_BackSpace, 0,
-			       "move_cursor", 3,
-			       G_TYPE_ENUM, GTK_MOVEMENT_PAGES,
-			       G_TYPE_INT, -1,
-			       G_TYPE_BOOLEAN, FALSE);
+			       "scroll_view", 1,
+			       GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_BACKWARD);
+  gtk_binding_entry_add_signal(binding_set, GDK_Page_Up, 0,
+			       "scroll_view", 1,
+			       GTK_TYPE_SCROLL_TYPE, GTK_SCROLL_PAGE_BACKWARD);
 
   gtk_binding_entry_add_signal(binding_set, GDK_S, GDK_CONTROL_MASK,
 			       "interactive_search", 1,
@@ -477,6 +489,7 @@ bbs_thread_view_init(BBSThreadView *view)
   view->last_offset = -1;
   view->thread = NULL;
   view->link_table = g_hash_table_new_full(NULL, NULL, NULL, TRACE_FREE);
+  view->child_table = g_hash_table_new(NULL, NULL);
 
   view->insert_mark = gtk_text_buffer_get_mark(buffer, "insert");
   view->selection_bound_mark = gtk_text_buffer_get_mark(buffer,
@@ -518,6 +531,12 @@ bbs_thread_view_destroy(GtkObject *object)
       view->link_table = NULL;
     }
 
+  if (view->child_table != NULL)
+    {
+      g_hash_table_destroy(view->child_table);
+      view->child_table = NULL;
+    }
+
   if (view->ibeam_cursor != NULL)
     {
       gdk_cursor_unref(view->ibeam_cursor);
@@ -1059,12 +1078,159 @@ bbs_thread_view_append_text_as_sage(BBSThreadView *view,
 void
 bbs_thread_view_append_widget(BBSThreadView *view, GtkWidget *widget)
 {
+  GtkWidget *vbox;
+  GtkWidget *hbox;
   GtkTextChildAnchor *anchor;
+  guint anchor_offset;
 
   g_return_if_fail(IS_BBS_THREAD_VIEW(view) && GTK_IS_WIDGET(widget));
 
+  vbox = gtk_vbox_new(FALSE, 0);
+  hbox = gtk_hbox_new(FALSE, 0);
+
+  gtk_widget_show(widget);
+  gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 30);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 20);
+  gtk_widget_show(vbox);
+
+  anchor_offset = gtk_text_iter_get_offset(&view->iter);
+
   anchor = gtk_text_buffer_create_child_anchor(view->text_buffer, &view->iter);
-  gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(view), widget, anchor);
+  gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(view), vbox, anchor);
+
+  anchor_offset = gtk_text_iter_get_offset(&view->iter);
+
+  g_hash_table_insert(view->child_table, widget, (gpointer)anchor_offset);
+}
+
+
+typedef struct _AdjustOffsetArgs
+{
+  gint offset;
+  GHashTable *hash_table;
+  GHashTable *new_hash_table;
+} AdjustOffsetArgs;
+
+
+void
+increment_values(gpointer key, gint value, AdjustOffsetArgs *args)
+{
+  if (value >= args->offset)
+    g_hash_table_replace(args->hash_table, key, (gpointer)value + 1);
+}
+
+
+gboolean
+update_link_table_increment(gint offset, char *url, AdjustOffsetArgs *args)
+{
+  if (offset >= args->offset)
+    g_hash_table_insert(args->new_hash_table, (gpointer)(offset + 1), url);
+  else
+    g_hash_table_insert(args->new_hash_table, (gpointer)offset, url);
+
+  return TRUE;
+}
+
+
+void
+bbs_thread_view_insert_widget_at_offset(BBSThreadView *view, GtkWidget *widget,
+					gint offset)
+{
+  GtkWidget *vbox;
+  GtkWidget *hbox;
+  GtkTextChildAnchor *anchor;
+  gint anchor_offset;
+  GtkTextIter iter;
+  AdjustOffsetArgs args;
+
+  g_return_if_fail(IS_BBS_THREAD_VIEW(view) && GTK_IS_WIDGET(widget));
+
+  vbox = gtk_vbox_new(FALSE, 0);
+  hbox = gtk_hbox_new(FALSE, 0);
+
+  gtk_widget_show(widget);
+  gtk_box_pack_start(GTK_BOX(hbox), widget, TRUE, TRUE, 30);
+  gtk_widget_show(hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 20);
+  gtk_widget_show(vbox);
+
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &iter, offset);
+
+  anchor = gtk_text_buffer_create_child_anchor(view->text_buffer, &iter);
+  gtk_text_view_add_child_at_anchor(GTK_TEXT_VIEW(view), vbox, anchor);
+
+  anchor_offset = gtk_text_iter_get_offset(&iter);
+
+  args.offset = offset;
+  args.hash_table = view->child_table;
+  g_hash_table_foreach(view->child_table, (GHFunc)increment_values, &args);
+
+  g_hash_table_insert(view->child_table, widget, (gpointer)anchor_offset);
+
+  args.hash_table = view->link_table;
+  args.new_hash_table = g_hash_table_new_full(NULL, NULL, NULL, TRACE_FREE);
+  g_hash_table_foreach_steal(view->link_table,
+			     (GHRFunc)update_link_table_increment, &args);
+  g_hash_table_destroy(view->link_table);
+  view->link_table = args.new_hash_table;
+
+  gtk_text_buffer_get_end_iter(view->text_buffer, &view->iter);
+}
+
+
+void
+decrement_values(gpointer key, gint value, AdjustOffsetArgs *args)
+{
+  if (value >= args->offset)
+    g_hash_table_replace(args->hash_table, key, (gpointer)value - 1);
+}
+
+
+gboolean
+update_link_table_decrement(gint offset, char *url, AdjustOffsetArgs *args)
+{
+  if (offset >= args->offset)
+    g_hash_table_insert(args->new_hash_table, (gpointer)(offset - 1), url);
+  else
+    g_hash_table_insert(args->new_hash_table, (gpointer)offset, url);
+
+  return TRUE;
+}
+
+
+void
+bbs_thread_view_remove_widget(BBSThreadView *view, GtkWidget *widget)
+{
+  GtkTextIter start;
+  GtkTextIter end;
+  gint anchor_offset;
+  AdjustOffsetArgs args;
+
+  g_return_if_fail(IS_BBS_THREAD_VIEW(view));
+  g_return_if_fail(GTK_IS_WIDGET(widget));
+
+  anchor_offset = (guint)g_hash_table_lookup(view->child_table, widget);
+  g_return_if_fail(anchor_offset != 0);
+
+  g_hash_table_remove(view->child_table, widget);
+
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &start,
+				     anchor_offset - 1);
+  gtk_text_buffer_get_iter_at_offset(view->text_buffer, &end, anchor_offset);
+  gtk_text_buffer_delete(view->text_buffer, &start, &end);
+  gtk_text_buffer_get_end_iter(view->text_buffer, &view->iter);
+
+  args.offset = anchor_offset - 1;
+  args.hash_table = view->child_table;
+  g_hash_table_foreach(view->child_table, (GHFunc)decrement_values, &args);
+
+  args.hash_table = view->link_table;
+  args.new_hash_table = g_hash_table_new_full(NULL, NULL, NULL, TRACE_FREE);
+  g_hash_table_foreach_steal(view->link_table,
+			     (GHRFunc)update_link_table_decrement, &args);
+  g_hash_table_destroy(view->link_table);
+  view->link_table = args.new_hash_table;
 }
 
 
@@ -1233,7 +1399,7 @@ bbs_thread_view_scroll_to_mark(BBSThreadView *view, GtkTextMark *mark)
   gtk_text_buffer_get_iter_at_mark(buffer, &iter, mark);
   gtk_text_buffer_place_cursor(buffer, &iter);
 
-  gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(view), mark, 0.2, TRUE, 0.0, 0.2);
+  gtk_text_view_scroll_to_mark(GTK_TEXT_VIEW(view), mark, 0.1, TRUE, 0.0, 0.1);
 }
 
 
