@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: board_properties.c,v 1.7 2003/11/28 03:06:53 fuyu Exp $
+ * $Id: board_properties.c,v 1.8 2003/12/12 21:43:52 fuyu Exp $
  */
 
 #include "config.h"
@@ -34,6 +34,7 @@
 #include "bulletin_board_ui.h"
 #include "board_properties.h"
 #include "boardlist_ui.h"
+#include "regex_utils.h"
 #include "threadlist_filter.h"
 
 #include <gtk/gtk.h>
@@ -52,13 +53,20 @@ typedef struct _DialogStates
   GtkToggleButton *show_mailto_button;
   GtkToggleButton *hide_mailto_button;
 
+  GtkToggleButton *board_local_a_bone_button;
+  GtkToggleButton *a_bone_by_name_button;
+  RegexEditor *a_bone_by_name_pattern;
+  GtkToggleButton *a_bone_by_id_button;
+  RegexEditor *a_bone_by_id_pattern;
+  GtkToggleButton *a_bone_by_content_button;
+  RegexEditor *a_bone_by_content_pattern;
+
   ThreadlistFilterSetting *setting;
   GtkToggleButton *use_as_default_button;
   GtkToggleButton *enable_filter_button;
 
   ThreadlistFilter *last_filter;
 
-  gboolean changed;
   gboolean use_default;
 } DialogStates;
 
@@ -80,7 +88,8 @@ static void restore_default_cb(ThreadlistFilterSetting *setting,
 static void apply_current_states(GtkWidget *dialog, DialogStates *states);
 static void board_properties_response_cb(GtkWidget *dialog, gint response_id,
 					 DialogStates *states);
-static void button_toggled_cb(GtkToggleButton *button, DialogStates *states);
+static void board_local_a_bone_toggled_cb(GtkToggleButton *button,
+					  DialogStates *states);
 
 static pthread_mutex_t board_properties_lock;
 
@@ -117,6 +126,8 @@ open_board_properties(OchushaApplication *application,
 {
   GtkWidget *dialog;
   GtkWidget *frame;
+  GtkWidget *notebook;
+  GtkWidget *vbox;
   GtkWidget *tmp_widget;
   DialogStates *states;
   BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
@@ -170,11 +181,41 @@ open_board_properties(OchushaApplication *application,
 
   frame = build_board_info_frame(application, board, states);
   gtk_widget_show(frame);
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), frame);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
+		     frame, FALSE, FALSE, 5);
+
+  notebook = gtk_notebook_new();
+  gtk_widget_show(notebook);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
+		     notebook, TRUE, TRUE, 0);
+
+
+  vbox = gtk_vbox_new(FALSE, 0);
+  gtk_widget_show(vbox);
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox,
+			   gtk_label_new(_("Threadlist's Filter Setting")));
+
+  frame = build_filter_setting_frame(application, board, states);
+  gtk_widget_show(frame);
+  gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
+
+  tmp_widget
+    = gtk_check_button_new_with_label(_("Apply filter to this board."));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_end(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
+  states->enable_filter_button = GTK_TOGGLE_BUTTON(tmp_widget);
+  gtk_toggle_button_set_active(states->enable_filter_button,
+			       info->enable_filter);
+
 
   frame = build_thread_view_setting_frame(application, board, states);
   gtk_widget_show(frame);
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), frame);
+
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), frame,
+			   gtk_label_new(_("Default Setting of Thread Views on This Board")));
+
+  g_signal_connect(G_OBJECT(states->board_local_a_bone_button), "toggled",
+		   G_CALLBACK(board_local_a_bone_toggled_cb), states);
 
   switch (info->show_mailto_mode)
     {
@@ -191,36 +232,62 @@ open_board_properties(OchushaApplication *application,
       break;
     }
 
-  frame = build_filter_setting_frame(application, board, states);
-  gtk_widget_show(frame);
-  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), frame);
+  if (info->board_local_a_bone)
+    {
+      gtk_toggle_button_set_active(states->a_bone_by_name_button,
+				   info->a_bone_by_name);
+      regex_editor_set_regex_text(states->a_bone_by_name_pattern,
+				  info->a_bone_by_name_pattern);
+      gtk_toggle_button_set_active(states->a_bone_by_id_button,
+				   info->a_bone_by_id);
+      regex_editor_set_regex_text(states->a_bone_by_id_pattern,
+				  info->a_bone_by_id_pattern);
+      gtk_toggle_button_set_active(states->a_bone_by_content_button,
+				   info->a_bone_by_content);
+      regex_editor_set_regex_text(states->a_bone_by_content_pattern,
+				  info->a_bone_by_content_pattern);
+    }
+  else
+    {
+      gtk_toggle_button_set_active(states->a_bone_by_name_button,
+				   application->a_bone_by_name);
+      regex_editor_set_regex_text(states->a_bone_by_name_pattern,
+				  application->a_bone_by_name_pattern);
+      gtk_toggle_button_set_active(states->a_bone_by_id_button,
+				   application->a_bone_by_id);
+      regex_editor_set_regex_text(states->a_bone_by_id_pattern,
+				  application->a_bone_by_id_pattern);
+      gtk_toggle_button_set_active(states->a_bone_by_content_button,
+				   application->a_bone_by_content);
+      regex_editor_set_regex_text(states->a_bone_by_content_pattern,
+				  application->a_bone_by_content_pattern);
+    }
 
-  tmp_widget
-    = gtk_check_button_new_with_label(_("Apply filter to this board."));
-  gtk_widget_show(tmp_widget);
-  gtk_box_pack_end(GTK_BOX(GTK_DIALOG(dialog)->vbox),
-		   tmp_widget, FALSE, FALSE, 0);
-  states->enable_filter_button = GTK_TOGGLE_BUTTON(tmp_widget);
-  g_signal_connect(G_OBJECT(tmp_widget), "toggled",
-		   G_CALLBACK(button_toggled_cb), states);
-  gtk_toggle_button_set_active(states->enable_filter_button,
-			       info->enable_filter);
+  gtk_toggle_button_set_active(states->board_local_a_bone_button,
+			       info->board_local_a_bone);
+  /* 礤ȥܤä */
+  board_local_a_bone_toggled_cb(states->board_local_a_bone_button, states);
 
-  states->changed = FALSE;
-  gtk_dialog_set_response_sensitive(GTK_DIALOG(dialog),
-				    GTK_RESPONSE_APPLY, FALSE);
   gtk_widget_show_all(dialog);
 }
 
 
 static void
-button_toggled_cb(GtkToggleButton *button, DialogStates *states)
+board_local_a_bone_toggled_cb(GtkToggleButton *button, DialogStates *states)
 {
-  if (!states->changed)
-    gtk_dialog_set_response_sensitive(
-				GTK_DIALOG(states->info->properties_dialog),
-				GTK_RESPONSE_APPLY, TRUE);
-  states->changed = TRUE;
+  gboolean sensitive = gtk_toggle_button_get_active(button);
+
+  gtk_widget_set_sensitive(GTK_WIDGET(states->a_bone_by_name_button),
+			   sensitive);
+  gtk_widget_set_sensitive(GTK_WIDGET(states->a_bone_by_name_pattern),
+			   sensitive);
+  gtk_widget_set_sensitive(GTK_WIDGET(states->a_bone_by_id_button), sensitive);
+  gtk_widget_set_sensitive(GTK_WIDGET(states->a_bone_by_id_pattern),
+			   sensitive);
+  gtk_widget_set_sensitive(GTK_WIDGET(states->a_bone_by_content_button),
+			   sensitive);
+  gtk_widget_set_sensitive(GTK_WIDGET(states->a_bone_by_content_pattern),
+			   sensitive);
 }
 
 
@@ -228,11 +295,6 @@ static void
 setting_changed_cb(ThreadlistFilterSetting *setting, ThreadlistFilter *filter,
 		   DialogStates *states)
 {
-  if (!states->changed)
-    gtk_dialog_set_response_sensitive(
-				GTK_DIALOG(states->info->properties_dialog),
-				GTK_RESPONSE_APPLY, TRUE);
-  states->changed = TRUE;
   states->use_default = FALSE;
 }
 
@@ -242,41 +304,69 @@ restore_default_cb(ThreadlistFilterSetting *setting, DialogStates *states)
 {
   threadlist_filter_setting_set_filter(setting,
 				&states->application->user_default_filter);
-  states->changed = TRUE;
   states->use_default = TRUE;
 
   states->info->filter.ignore_threshold
     = states->application->user_default_filter.ignore_threshold;
   gtk_toggle_button_set_active(states->use_as_default_button, TRUE);
-  gtk_dialog_set_response_sensitive(
-				GTK_DIALOG(states->info->properties_dialog),
-				GTK_RESPONSE_APPLY, TRUE);
 }
 
 
 static void
 apply_current_states(GtkWidget *dialog, DialogStates *states)
 {
+  BulletinBoardGUIInfo *info = states->info;
   const ThreadlistFilter *filter;
 
-  if (!states->changed)
-    return;
-
   filter = threadlist_filter_setting_get_filter(states->setting);
-  states->info->enable_filter
+  info->enable_filter
     = gtk_toggle_button_get_active(states->enable_filter_button);
 
   if (gtk_toggle_button_get_active(states->show_mailto_button))
     {
-      states->info->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_SHOW;
+      info->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_SHOW;
     }
   else if (gtk_toggle_button_get_active(states->hide_mailto_button))
     {
-      states->info->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_HIDE;
+      info->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_HIDE;
     }
   else
     {
-      states->info->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_DEFAULT;
+      info->show_mailto_mode = THREAD_VIEW_MAILTO_MODE_DEFAULT;
+    }
+
+  info->board_local_a_bone
+    = gtk_toggle_button_get_active(states->board_local_a_bone_button);
+
+  if (info->a_bone_by_name_pattern != NULL)
+    G_FREE(info->a_bone_by_name_pattern);
+
+  if (info->a_bone_by_id_pattern != NULL)
+    G_FREE(info->a_bone_by_id_pattern);
+
+  if (info->a_bone_by_content_pattern != NULL)
+    G_FREE(info->a_bone_by_content_pattern);
+
+  if (info->board_local_a_bone)
+    {
+      info->a_bone_by_name
+	= gtk_toggle_button_get_active(states->a_bone_by_name_button);
+      info->a_bone_by_name_pattern
+	= regex_editor_get_regex_text(states->a_bone_by_name_pattern);
+      info->a_bone_by_id
+	= gtk_toggle_button_get_active(states->a_bone_by_id_button);
+      info->a_bone_by_id_pattern
+	= regex_editor_get_regex_text(states->a_bone_by_id_pattern);
+      info->a_bone_by_content
+	= gtk_toggle_button_get_active(states->a_bone_by_content_button);
+      info->a_bone_by_content_pattern
+	= regex_editor_get_regex_text(states->a_bone_by_content_pattern);
+    }
+  else
+    {
+      info->a_bone_by_name_pattern = NULL;
+      info->a_bone_by_id_pattern = NULL;
+      info->a_bone_by_content_pattern = NULL;
     }
 
   if (gtk_toggle_button_get_active(states->use_as_default_button))
@@ -286,14 +376,14 @@ apply_current_states(GtkWidget *dialog, DialogStates *states)
       states->application->user_default_filter.ignore_threshold
 	= filter->ignore_threshold;
 
-      if (states->info->filter.rule != NULL)
+      if (info->filter.rule != NULL)
 	{
-	  gchar *tmp = states->info->filter.rule;
-	  states->info->filter.rule = NULL;
+	  gchar *tmp = info->filter.rule;
+	  info->filter.rule = NULL;
 	  G_FREE(tmp);
 	}
 
-      states->info->filter.ignore_threshold = filter->ignore_threshold;
+      info->filter.ignore_threshold = filter->ignore_threshold;
       threadlist_filter_setting_set_restore_default_button_sensitive(
 						states->setting, FALSE);
       setup_default_threadlist_filter(states->application);
@@ -301,27 +391,22 @@ apply_current_states(GtkWidget *dialog, DialogStates *states)
     }
   else
     {
-      if (states->info->filter.rule != NULL)
+      if (info->filter.rule != NULL)
 	{
-	  gchar *tmp = states->info->filter.rule;
-	  states->info->filter.rule = NULL;
+	  gchar *tmp = info->filter.rule;
+	  info->filter.rule = NULL;
 	  G_FREE(tmp);
 	}
 
       if (!states->use_default)
 	{
-	  states->info->filter.rule = G_STRDUP(filter->rule);
-	  states->info->filter.ignore_threshold = filter->ignore_threshold;
+	  info->filter.rule = G_STRDUP(filter->rule);
+	  info->filter.ignore_threshold = filter->ignore_threshold;
 	}
 
       redraw_threadlist_view(states->application, states->board);
       fix_filter_button_state(states->application);
     }
-
-  states->changed = FALSE;
-  gtk_dialog_set_response_sensitive(
-				GTK_DIALOG(states->info->properties_dialog),
-				GTK_RESPONSE_APPLY, FALSE);
 }
 
 
@@ -407,10 +492,14 @@ build_thread_view_setting_frame(OchushaApplication *application,
 {
   GtkWidget *tmp_widget;
   GtkWidget *thread_frame = gtk_frame_new(_("Default Setting of Thread Views on This Board"));
+  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
   GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
 
+  gtk_widget_show(vbox);
+  gtk_container_add(GTK_CONTAINER(thread_frame), vbox);
+
   gtk_widget_show(hbox);
-  gtk_container_add(GTK_CONTAINER(thread_frame), hbox);
+  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
   tmp_widget = gtk_label_new(_("Show mailto literally:"));
   gtk_widget_show(tmp_widget);
@@ -418,28 +507,57 @@ build_thread_view_setting_frame(OchushaApplication *application,
 
   tmp_widget
     = gtk_radio_button_new_with_label(NULL, _("Use Ochusha's Default"));
-  g_signal_connect(G_OBJECT(tmp_widget), "toggled",
-		   G_CALLBACK(button_toggled_cb), states);
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
   states->use_ochusha_default_button = GTK_TOGGLE_BUTTON(tmp_widget);
   
   tmp_widget = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(states->use_ochusha_default_button), _("Show mailto"));
-  g_signal_connect(G_OBJECT(tmp_widget), "toggled",
-		   G_CALLBACK(button_toggled_cb), states);
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
   states->show_mailto_button = GTK_TOGGLE_BUTTON(tmp_widget);
 
   tmp_widget = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(states->use_ochusha_default_button), _("Hide mailto"));
-  g_signal_connect(G_OBJECT(tmp_widget), "toggled",
-		   G_CALLBACK(button_toggled_cb), states);
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(hbox), tmp_widget, FALSE, FALSE, 0);
   states->hide_mailto_button = GTK_TOGGLE_BUTTON(tmp_widget);
 
   gtk_widget_show(hbox);
 
+  tmp_widget = gtk_check_button_new_with_label(_("Use board specific setting for a bone."));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
+  states->board_local_a_bone_button = GTK_TOGGLE_BUTTON(tmp_widget);
+
+  tmp_widget = gtk_check_button_new_with_label(_("A bone by name field."));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
+  states->a_bone_by_name_button = GTK_TOGGLE_BUTTON(tmp_widget);
+
+  tmp_widget = regex_editor_new(NULL);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, TRUE, TRUE, 0);
+  states->a_bone_by_name_pattern = REGEX_EDITOR(tmp_widget);
+
+  tmp_widget = gtk_check_button_new_with_label(_("A bone by ID."));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
+  states->a_bone_by_id_button = GTK_TOGGLE_BUTTON(tmp_widget);
+
+  tmp_widget = regex_editor_new(NULL);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, TRUE, TRUE, 0);
+  states->a_bone_by_id_pattern = REGEX_EDITOR(tmp_widget);
+
+  tmp_widget = gtk_check_button_new_with_label(_("A bone by content."));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
+  states->a_bone_by_content_button = GTK_TOGGLE_BUTTON(tmp_widget);
+
+  tmp_widget = regex_editor_new(NULL);
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, TRUE, TRUE, 0);
+  states->a_bone_by_content_pattern = REGEX_EDITOR(tmp_widget);
+
   return thread_frame;
 }
 
@@ -461,8 +579,6 @@ build_filter_setting_frame(OchushaApplication *application,
 
   tmp_widget = gtk_check_button_new_with_label(_("Use this setting as the default setting."));
   states->use_as_default_button = GTK_TOGGLE_BUTTON(tmp_widget);
-  g_signal_connect(G_OBJECT(tmp_widget), "toggled",
-		   G_CALLBACK(button_toggled_cb), states);
   gtk_widget_show(tmp_widget);
   gtk_box_pack_start(GTK_BOX(vbox), tmp_widget, FALSE, FALSE, 0);
 
