@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_ui.c,v 1.38 2003/12/16 18:00:54 fuyu Exp $
+ * $Id: boardlist_ui.c,v 1.40 2003/12/19 03:14:24 fuyu Exp $
  */
 
 #include "config.h"
@@ -144,10 +144,10 @@ static GtkWidget *create_new_thread_dialog(OchushaApplication *application,
 /* Υå˰٤ǤⳫ(OchushaBulletinBoard *)Υꥹȡ*/
 static GSList *board_list;
 
-static pthread_mutex_t category_info_lock;
 static pthread_mutex_t board_list_lock;
 
-
+#if 0	/* Ϥפäݤ*/
+static pthread_mutex_t category_info_lock;
 #define CATEGORY_INFO_LOCK				\
   if (pthread_mutex_lock(&category_info_lock) != 0)	\
     {							\
@@ -161,6 +161,7 @@ static pthread_mutex_t board_list_lock;
       fprintf(stderr, "Couldn't unlock a mutex.\n");	\
       abort();						\
     }
+#endif
 
 
 #define BOARD_LIST_LOCK					\
@@ -191,6 +192,14 @@ static GQuark application_id;
 void
 prepare_boardlist_ui_initialization(OchushaApplication *application)
 {
+#ifdef CATEGORY_INFO_LOCK
+  if (pthread_mutex_init(&category_info_lock, NULL) != 0)
+    {
+      fprintf(stderr, "Couldn't init a mutex.\n");
+      abort();
+    }
+#endif
+
   if (pthread_mutex_init(&board_list_lock, NULL) != 0)
     {
       fprintf(stderr, "Couldn't init a mutex.\n");
@@ -228,6 +237,9 @@ open_threadlist_view_cb(GtkWidget *selector, OchushaBulletinBoard *board,
 }
 
 
+static guint new_thread_netstat_id;
+
+
 void
 initialize_boardlist(OchushaApplication *application)
 {
@@ -237,6 +249,9 @@ initialize_boardlist(OchushaApplication *application)
 
   board_list = NULL;
 
+  new_thread_netstat_id = gtk_statusbar_get_context_id(application->statusbar,
+						       "new-thread-netstat");
+
   application->boardlist_view = boardlist_view_new();
   g_object_set_qdata(G_OBJECT(application->boardlist_view), application_id,
 		     application);
@@ -400,6 +415,16 @@ trace_free(gpointer pointer)
 static BoardCategoryGUIInfo *
 ensure_board_category_info(OchushaBoardCategory *category)
 {
+#ifndef CATEGORY_INFO_LOCK
+  BoardCategoryGUIInfo *info = g_object_get_qdata(G_OBJECT(category),
+						  category_gui_info_id);
+  if (info == NULL)
+    {
+      info = G_NEW0(BoardCategoryGUIInfo, 1);
+      g_object_set_qdata_full(G_OBJECT(category), category_gui_info_id,
+			      info, (GDestroyNotify)TRACE_FREE);
+    }
+#else
   BoardCategoryGUIInfo *info;
 
   CATEGORY_INFO_LOCK
@@ -413,6 +438,7 @@ ensure_board_category_info(OchushaBoardCategory *category)
       }
   }
   CATEGORY_INFO_UNLOCK;
+#endif
 
   return info;
 }
@@ -837,7 +863,7 @@ start_current_threadlist_search(OchushaApplication *application)
 
 
 /*
- * ϤʤΤѰդषOchushaBulletinBoardѲưŪ
+ * ϤʤΤѰդषBulletinBoardGUIInfoѲưŪ
  * ˻ΤƼΧŪб٤ʤΤʡ
  */
 void
@@ -1177,11 +1203,17 @@ boardlist_read_board_element_cb(OchushaBBSTable *table,
 
 
 #define OUTPUT_CATEGORY_ATTRIBUTE_BOOLEAN(file, category, attribute)	\
-  fprintf(file,								\
-	  "      <attribute name=\"" #attribute	"\">\n"			\
-	  "        <boolean val=\"%s\"/>\n"				\
-	  "      </attribute>\n",					\
-	  (category)->attribute ? "true" : "false")
+  do									\
+    {									\
+      if ((category)->attribute)					\
+	{								\
+	  fprintf(file,							\
+		  "      <attribute name=\"" #attribute	"\">\n"		\
+		  "        <boolean val=\"%s\"/>\n"			\
+		  "      </attribute>\n",				\
+		  (category)->attribute ? "true" : "false");		\
+	}								\
+    } while (0)
 
 
 static void
@@ -1197,17 +1229,31 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
 
 
 #define OUTPUT_BOARD_ATTRIBUTE_BOOLEAN(file, board, attribute)		\
-  fprintf(file,								\
-	  "        <attribute name=\"" #attribute	"\">\n"		\
-	  "          <boolean val=\"%s\"/>\n"				\
-	  "        </attribute>\n",					\
-	  (board)->attribute ? "true" : "false")
+  do									\
+    {									\
+      if ((board)->attribute)						\
+	{								\
+	  fprintf(file,							\
+		  "        <attribute name=\"" #attribute "\">\n"	\
+		  "          <boolean val=\"%s\"/>\n"			\
+		  "        </attribute>\n",				\
+		  (board)->attribute ? "true" : "false");		\
+	}								\
+    } while (0)
+
 
 #define OUTPUT_BOARD_ATTRIBUTE_INT(file, board, attribute)		\
-  fprintf(file,								\
-	  "        <attribute name=\"" #attribute	"\">\n"		\
-	  "          <int val=\"%d\"/>\n"				\
-	  "        </attribute>\n", (board)->attribute)
+  do									\
+    {									\
+      if ((board)->attribute)						\
+	{								\
+	  fprintf(file,							\
+		  "        <attribute name=\"" #attribute "\">\n"	\
+		  "          <int val=\"%d\"/>\n"			\
+		  "        </attribute>\n", (board)->attribute);	\
+	}								\
+    } while (0)
+
 
 #define OUTPUT_BOARD_ATTRIBUTE_STRING(file, board, attribute)		\
   do {									\
@@ -1215,7 +1261,7 @@ boardlist_write_category_element_cb(OchushaBBSTable *table,
       {									\
 	gchar *text = g_markup_escape_text((board)->attribute, -1);	\
 	fprintf(file,							\
-		"        <attribute name=\"" #attribute	"\">\n"		\
+		"        <attribute name=\"" #attribute "\">\n"		\
 		"          <string>%s</string>\n"			\
 		"        </attribute>\n", text);			\
 	g_free(text);							\
@@ -1515,6 +1561,14 @@ new_thread_dialog_response_cb(GtkWidget *dialog, gint response_id,
 {
   OchushaBulletinBoard *board = g_object_get_data(G_OBJECT(dialog), "board");
   gboolean result;
+  guint message_id = (guint)g_object_get_data(G_OBJECT(dialog),
+					      "statusbar_message");
+  if (message_id != 0)
+    {
+      gtk_statusbar_remove(application->statusbar,
+			   new_thread_netstat_id, message_id);
+      g_object_set_data(G_OBJECT(dialog), "statusbar_message", NULL);
+    }
 
   if (response_id == GTK_RESPONSE_OK)
     {
@@ -1532,6 +1586,14 @@ new_thread_dialog_response_cb(GtkWidget *dialog, gint response_id,
       if (title != NULL && *title != '\0'
 	  && message != NULL && *message != '\0')
 	{
+	  gchar statusbar_message[4096];
+
+	  snprintf(statusbar_message, 4096,
+		   _("Creating new thread at %s board."), board->name);
+	  message_id = gtk_statusbar_push(application->statusbar,
+					  new_thread_netstat_id,
+					  statusbar_message);
+
 	  result = ochusha_bulletin_board_create_new_thread(board,
 						application->broker,
 						title, &response);
@@ -1541,6 +1603,9 @@ new_thread_dialog_response_cb(GtkWidget *dialog, gint response_id,
 	      BulletinBoardGUIInfo *board_info
 		= ensure_bulletin_board_info(board, application);
 
+	      gtk_statusbar_remove(application->statusbar,
+				   new_thread_netstat_id, message_id);
+
 	      if (board_info->last_name == NULL
 		  || strcmp(board_info->last_name, response.name) != 0)
 		{
@@ -1573,6 +1638,19 @@ new_thread_dialog_response_cb(GtkWidget *dialog, gint response_id,
 		  application->last_mail = G_STRDUP(response.mailto);
 		}
 	    }
+	  else
+	    {
+	      gtk_statusbar_remove(application->statusbar,
+				   new_thread_netstat_id, message_id);
+	      snprintf(statusbar_message, 4096,
+		       _("Creation of new thread at %s board failed."),
+		       board->name);
+	      message_id = gtk_statusbar_push(application->statusbar,
+					      new_thread_netstat_id,
+					      statusbar_message);
+	      g_object_set_data(G_OBJECT(dialog), "statusbar_message",
+				(gpointer)message_id);
+	    }
 	}
       else
 	{
