@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.15 2003/12/05 09:21:52 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.16 2003/12/16 14:50:33 fuyu Exp $
  */
 
 #include "config.h"
@@ -125,6 +125,7 @@ enum {
   TOGGLE_HIDE_BOARD_SIGNAL,
   KILL_CATEGORY_SIGNAL,
   KILL_BOARD_SIGNAL,
+  COPY_BOARD_URL_SIGNAL,
 
   ADVANCE_VIEW_SIGNAL,
   BACK_VIEW_SIGNAL,
@@ -149,13 +150,14 @@ enum {
   POPUP_MENU_OPEN_SELECTION_IN_TAB,
   POPUP_MENU_OPEN_SELECTION_WITH_BROWSER,
   POPUP_MENU_TOGGLE_HIDE_SELECTION,
-  POPUP_MENU_KILL_SELECTION
+  POPUP_MENU_KILL_SELECTION,
+  POPUP_MENU_COPY_BOARD_URL
 };
 
 
 static GtkTreeViewClass *parent_class = NULL;
 static gint boardlist_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GQuark node_info_id;
 
@@ -241,7 +243,21 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 	popup_menu_callback,			/* callback_func */
 	POPUP_MENU_KILL_SELECTION,		/* act */
 	NULL					/* type */
-      }
+      },
+      {
+	_("/-------"),
+	NULL,
+	NULL,
+	0,
+	"<Separator>"
+      },
+      {
+	_("/Copy Board's URL"),			/* menu path */
+	NULL,					/* accelerator */
+	popup_menu_callback,			/* callback_func */
+	POPUP_MENU_COPY_BOARD_URL,		/* act */
+	NULL					/* type */
+      },
     };
   gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
 
@@ -362,6 +378,15 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 		 ochusha_marshal_VOID__OBJECT,
 		 G_TYPE_NONE, 1,
 		 OCHUSHA_TYPE_BULLETIN_BOARD);
+  boardlist_view_signals[COPY_BOARD_URL_SIGNAL] =
+    g_signal_new("copy_board_url",
+		 G_TYPE_FROM_CLASS(klass),
+		 G_SIGNAL_RUN_LAST,
+		 G_STRUCT_OFFSET(BoardlistViewClass, copy_board_url),
+		 NULL, NULL,
+		 ochusha_marshal_VOID__OBJECT,
+		 G_TYPE_NONE, 1,
+		 OCHUSHA_TYPE_BULLETIN_BOARD);
 
   boardlist_view_signals[ADVANCE_VIEW_SIGNAL] =
     g_signal_new("advance_view",
@@ -459,6 +484,7 @@ boardlist_view_class_init(BoardlistViewClass *klass)
   klass->toggle_hide_board = NULL;
   klass->kill_category = NULL;
   klass->kill_board = NULL;
+  klass->copy_board_url = NULL;
 
   klass->advance_view = NULL;
   klass->back_view = NULL;
@@ -731,6 +757,14 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
 			     -1);
 	}
       break;
+
+    case POPUP_MENU_COPY_BOARD_URL:
+      if (OCHUSHA_IS_BULLETIN_BOARD(data))
+	g_signal_emit(G_OBJECT(args->view),
+		      boardlist_view_signals[COPY_BOARD_URL_SIGNAL],
+		      0,
+		      data);
+      break;
     }
 }
 
