@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.h,v 1.8 2003/12/03 07:53:11 fuyu Exp $
+ * $Id: boardlist_view.h,v 1.9 2003/12/16 14:50:33 fuyu Exp $
  */
 
 #ifndef _BOARDLIST_VIEW_H_
@@ -88,6 +88,8 @@ struct _BoardlistViewClass
 			OchushaBoardCategory *category);
   void (*kill_board)(BoardlistView *boardlist_view,
 		     OchushaBulletinBoard *board);
+  void (*copy_board_url)(BoardlistView *boardlist_view,
+			 OchushaBulletinBoard *board);
 
   gboolean (*advance_view)(BoardlistView *boardlist_view);
   gboolean (*back_view)(BoardlistView *boardlist_view);
