@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.73 2003/12/16 18:00:54 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.79 2003/12/20 04:20:18 fuyu Exp $
  */
 
 #include "config.h"
@@ -61,6 +61,7 @@
 
 
 static void open_thread_view(WorkerThread *employee, gpointer job_args);
+static void open_kako_thread_view(WorkerThread *employee, gpointer job_args);
 static void show_threadlist(WorkerThread *employee, gpointer args);
 static void advance_view_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			    PanedNotebook *paned_notebook);
@@ -116,7 +117,7 @@ static void refresh_thread_button_cb(GtkWidget *widget,
 				     OchushaApplication *application);
 static void start_thread_search_button_cb(GtkWidget *widget,
 					  OchushaApplication *application);
-static void jump_to_new_comer_button_cb(GtkWidget *widget,
+static void jump_to_bookmark_button_cb(GtkWidget *widget,
 					OchushaApplication *application);
 static void go_to_bottom_button_cb(GtkWidget *widget,
 				   OchushaApplication *application);
@@ -126,10 +127,11 @@ static GtkWidget *create_write_dialog(OchushaApplication *application,
 				      OchushaBBSThread *thread);
 
 
-static pthread_mutex_t board_info_lock;
 static pthread_mutex_t thread_list_lock;
 static pthread_cond_t thread_list_condition;
 
+#if 0
+static pthread_mutex_t board_info_lock;
 #define BOARD_INFO_LOCK					\
   if (pthread_mutex_lock(&board_info_lock) != 0)	\
     {							\
@@ -143,6 +145,7 @@ static pthread_cond_t thread_list_condition;
       fprintf(stderr, "Couldn't unlock a mutex.\n");	\
       abort();						\
     }
+#endif
 
 #define THREAD_LIST_LOCK				\
   if (pthread_mutex_lock(&thread_list_lock) != 0)	\
@@ -192,11 +195,13 @@ static guint writing_netstat_id;
 void
 prepare_board_ui_initialization(OchushaApplication *application)
 {
+#ifdef BOARD_INFO_LOCK
  if (pthread_mutex_init(&board_info_lock, NULL) != 0)
     {
       fprintf(stderr, "Couldn't init a mutex.\n");
       abort();
     }
+#endif
 
  if (pthread_mutex_init(&thread_list_lock, NULL) != 0)
     {
@@ -273,6 +278,11 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
   if ((info->view_flags & BBS_THREAD_HIDDEN) != 0)
     thread->number_of_responses_read = 0;
 
+  info->bookmark_response_number
+    = ochusha_utils_get_attribute_int(hash_table, "bookmark_response_number");
+  if (info->bookmark_response_number == 0)
+    info->bookmark_response_number = thread->number_of_responses_read;
+
   info->thread_local_a_bone
     = ochusha_utils_get_attribute_boolean(hash_table, "thread_local_a_bone");
   info->a_bone_by_name
@@ -294,16 +304,31 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
 
 
 #define OUTPUT_THREAD_ATTRIBUTE_BOOLEAN(file, thread, attribute)	\
-  fprintf(file,								\
-	  "      <attribute name=\"" #attribute	"\">\n"			\
-	  "        <boolean val=\"%s\"/>\n"				\
-	  "      </attribute>\n", (thread)->attribute ? "true" : "false")
+  do									\
+    {									\
+      if ((thread)->attribute)						\
+	{								\
+	  fprintf(file,							\
+		  "      <attribute name=\"" #attribute	"\">\n"		\
+		  "        <boolean val=\"%s\"/>\n"			\
+		  "      </attribute>\n",				\
+		  (thread)->attribute ? "true" : "false");		\
+	}								\
+    } while (0)
+
 
 #define OUTPUT_THREAD_ATTRIBUTE_INT(file, thread, attribute)		\
-  fprintf(file,								\
-	  "      <attribute name=\"" #attribute	"\">\n"			\
-	  "        <int val=\"%d\"/>\n"					\
-	  "      </attribute>\n", (thread)->attribute)
+  do									\
+    {									\
+      if ((thread)->attribute)						\
+	{								\
+	  fprintf(file,							\
+		  "      <attribute name=\"" #attribute	"\">\n"		\
+		  "        <int val=\"%d\"/>\n"				\
+		  "      </attribute>\n", (thread)->attribute);		\
+	}								\
+    } while (0)
+
 
 #define OUTPUT_THREAD_ATTRIBUTE_STRING(file, thread, attribute)		\
   do {									\
@@ -338,6 +363,8 @@ threadlist_write_thread_element_cb(OchushaBulletinBoard *board,
       OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info, view_ignored);
       OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, info, last_name);
       OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, info, last_mail);
+      OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info,
+				  bookmark_response_number);
     }
 
   if ((info->view_flags & BBS_THREAD_HIDDEN) != 0)
@@ -413,8 +440,25 @@ BulletinBoardGUIInfo *
 ensure_bulletin_board_info(OchushaBulletinBoard *board,
 			   OchushaApplication *application)
 {
+#ifndef BOARD_INFO_LOCK
+  BulletinBoardGUIInfo *info = g_object_get_qdata(G_OBJECT(board),
+						  board_info_id);
+  if (info == NULL)
+    {
+      info = G_NEW0(BulletinBoardGUIInfo, 1);
+      g_object_set_qdata_full(G_OBJECT(board), board_info_id, info,
+			      (GDestroyNotify)bulletin_board_gui_info_free);
+      g_signal_connect(G_OBJECT(board), "threadlist_read_thread_element",
+		       G_CALLBACK(threadlist_read_thread_element_cb), NULL);
+      g_signal_connect(G_OBJECT(board), "threadlist_write_thread_element",
+		       G_CALLBACK(threadlist_write_thread_element_cb),
+		       application);
+      info->recover_mode = application->recover_mode;
+    }
+#else
   BulletinBoardGUIInfo *info;
-  BOARD_INFO_LOCK
+
+  BOARD_INFO_LOCK 
   {
     info = g_object_get_qdata(G_OBJECT(board), board_info_id);
     if (info == NULL)
@@ -431,6 +475,7 @@ ensure_bulletin_board_info(OchushaBulletinBoard *board,
       }
   }
   BOARD_INFO_UNLOCK;
+#endif
 
   return info;
 }
@@ -553,13 +598,25 @@ customize_threadlist_view_contents(OchushaApplication *application,
 	  break;
 
 	case 'n':
-	  threadlist_view_append_number_of_responses_read(view);
+	  threadlist_view_append_number_of_responses_got(view);
 	  break;
 
 	case 'U':
+	  threadlist_view_append_number_of_responses_ungot(view);
+	  break;
+
+	case 'u':
 	  threadlist_view_append_number_of_responses_unread(view);
 	  break;
 
+	case 'S':
+	  threadlist_view_append_number_of_responses_shown(view);
+	  break;
+
+	case 'V':
+	  threadlist_view_append_number_of_responses_verbose(view);
+	  break;
+
 	case 'D':
 	  threadlist_view_append_rank_difference(view);
 	  break;
@@ -701,9 +758,9 @@ open_bulletin_board(OchushaApplication *application,
 			     application,
 			     0);
     gtk_toolbar_insert_stock(toolbar, GTK_STOCK_JUMP_TO,
-			     _("Jump to new comer response"),
-			     "jump_to_new_comer_response",
-			     GTK_SIGNAL_FUNC(jump_to_new_comer_button_cb),
+			     _("Jump to bookmark"),
+			     "jump_to_bookmark",
+			     GTK_SIGNAL_FUNC(jump_to_bookmark_button_cb),
 			     application,
 			     0);
     gtk_toolbar_insert_stock(toolbar, GTK_STOCK_GOTO_BOTTOM,
@@ -1047,6 +1104,102 @@ redraw_threadlist_view(OchushaApplication *application,
 }
 
 
+typedef struct _OpenKakoThreadJobArgs
+{
+  OchushaApplication *application;
+  OchushaBulletinBoard *board;
+
+  char *url;
+
+  gboolean in_tab;
+} OpenKakoThreadJobArgs;
+
+
+void
+ochusha_open_kako_thread(OchushaApplication *application,
+			 OchushaBulletinBoard *board, const char *url,
+			 gboolean in_tab)
+{
+  WorkerJob *job;
+  OpenKakoThreadJobArgs *args;
+
+  if (board == NULL)
+    return;
+
+  paned_notebook_open_item_view(PANED_NOTEBOOK(application->contents_window),
+				board, in_tab);
+  /*
+   * Ĥ򳫤Τ̥åɤǹԤǽꡢ
+   * Ǥϡ򳫤ΤϤθˤʤΤǡĤΤλ
+   * Ԥθ她򳫤ΥåɤȤ
+   */
+  job = G_NEW0(WorkerJob, 1);
+  args = G_NEW0(OpenKakoThreadJobArgs, 1);
+
+  job->canceled = FALSE;
+  job->job = open_kako_thread_view;
+  job->args = args;
+
+  args->application = application;
+  args->board = board;
+
+  args->url = G_STRDUP(url);
+
+  args->in_tab = in_tab;
+
+  commit_job(job);
+}
+
+
+static void
+open_kako_thread_view(WorkerThread *employee, gpointer job_args)
+{
+  OpenKakoThreadJobArgs *args = (OpenKakoThreadJobArgs *)job_args;
+  OchushaApplication *application = args->application;
+  OchushaBulletinBoard *board = args->board;
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
+
+  THREAD_LIST_LOCK
+  {
+    PanedNotebook *board_view;
+    OchushaBBSThread *thread
+      = ochusha_bulletin_board_lookup_bbs_thread_by_url(board, args->url);
+
+    /* ޤġġ*/
+    while (info->is_busy && thread == NULL)
+      {
+	THREAD_LIST_COND_WAIT;
+	thread = ochusha_bulletin_board_lookup_bbs_thread_by_url(board,
+								 args->url);
+      }
+
+    if (thread == NULL)
+      thread = ochusha_bulletin_board_lookup_kako_thread_by_url(board,
+							application->broker,
+							args->url);
+
+#if DEBUG_GUI_MOST
+    fprintf(stderr, "Kako Thread URL: \"%s\"\n", args->url);
+#endif
+    /* λǡ٤Ϻ줿Ȥݾڤ */
+
+    if (thread != NULL)
+      {
+	gdk_threads_enter();
+	board_view = (PanedNotebook *)paned_notebook_get_item_view(PANED_NOTEBOOK(application->contents_window), board);
+
+	if (board_view != NULL)
+	  paned_notebook_open_item_view(board_view, thread, args->in_tab);
+	gdk_threads_leave();
+      }
+  }
+  THREAD_LIST_UNLOCK;
+
+  G_FREE(args->url);
+  G_FREE(args);
+}
+
+
 typedef struct _EachThreadArgs
 {
   OchushaApplication *application;
@@ -1099,7 +1252,10 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
     {
       gdk_threads_enter();
 
-      threadlist_view_append_thread(view, thread,
+      threadlist_view_append_thread(view, thread, info->num_shown,
+				    (MAX(thread->number_of_responses_on_server,
+					 thread->number_of_responses_read)
+				     - info->bookmark_response_number),
 				    info->view_rank, args->number_of_threads);
       update_threadlist_entry_style(args->application, view, thread);
 
@@ -1170,7 +1326,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
   if (job_args->increment_ignored)
     {
       gchar message[4096];
-      snprintf(message, 4096, _("Updating threadlist of %s board"),
+      snprintf(message, 4096, _("Updating threadlist of %s board."),
 	       board->name);
       if (info->message_id != 0)
 	gtk_statusbar_remove(application->statusbar, threadlist_netstat_id,
@@ -1319,7 +1475,10 @@ update_threadlist_entry_style(OchushaApplication *application,
 	visual_parameter.background_color = LIST_ENTRY_BACKGROUND_EMPH;
     }
 
-  threadlist_view_update_thread(view, thread);
+  threadlist_view_update_thread(view, thread, info->num_shown,
+				(MAX(thread->number_of_responses_on_server,
+				     thread->number_of_responses_read)
+				 - info->bookmark_response_number));
   threadlist_view_update_thread_visual(view, thread, &visual_parameter);
 }
 
@@ -1335,10 +1494,8 @@ advance_view_cb(ThreadlistView *view, OchushaBBSThread *thread,
       /* ڡ򥹥뤵 */
       GtkWidget *scrolled_window
 	= paned_notebook_get_current_item_view(paned_notebook);
-      BBSThreadView *thread_view
-	= BBS_THREAD_VIEW(gtk_bin_get_child(GTK_BIN(scrolled_window)));
-      g_signal_emit_by_name(G_OBJECT(thread_view), "move_cursor",
-			    GTK_MOVEMENT_PAGES, 1, FALSE);
+      g_signal_emit_by_name(G_OBJECT(scrolled_window), "scroll_child",
+			    GTK_SCROLL_PAGE_FORWARD, FALSE);
     }
 }
 
@@ -1354,10 +1511,8 @@ back_view_cb(ThreadlistView *view, OchushaBBSThread *thread,
       /* ڡ򥹥뤵 */
       GtkWidget *scrolled_window
 	= paned_notebook_get_current_item_view(paned_notebook);
-      BBSThreadView *thread_view
-	= BBS_THREAD_VIEW(gtk_bin_get_child(GTK_BIN(scrolled_window)));
-      g_signal_emit_by_name(G_OBJECT(thread_view), "move_cursor",
-			    GTK_MOVEMENT_PAGES, -1, FALSE);
+      g_signal_emit_by_name(G_OBJECT(scrolled_window), "scroll_child",
+			    GTK_SCROLL_PAGE_BACKWARD, FALSE);
     }
 }
 
@@ -1606,6 +1761,7 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
 {
   BBSThreadGUIInfo *info;
   OchushaAsyncBuffer *buffer;
+  GtkWidget *threadlist_view;
 
   g_return_if_fail(GTK_IS_WIDGET(item_view));
 
@@ -1651,6 +1807,8 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
       gdk_threads_enter();
     }
 
+  threadlist_view = gtk_bin_get_child(GTK_BIN(paned_notebook->selector));
+
   if (item_view != NULL)
     {
       GtkWidget *thread_view = gtk_bin_get_child(GTK_BIN(item_view));
@@ -1660,17 +1818,25 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook,
       if (paned_notebook->number_of_items_shown == 1)
 	{
 	  if (gtk_widget_is_focus(thread_view))
-	    {
-	      GtkWidget *threadlist_view
-		= gtk_bin_get_child(GTK_BIN(paned_notebook->selector));
-	      gtk_widget_grab_focus(GTK_WIDGET(threadlist_view));
-	    }
+	    gtk_widget_grab_focus(threadlist_view);
 	}
     }
 
   info->res_num = 0;
+  info->num_shown = 0;
   info->last_read = NULL;
   info->next_mark = NULL;
+  info->bookmark = NULL;
+
+  if (!application->stick_bookmark)
+    info->bookmark_response_number = thread->number_of_responses_read;
+
+  threadlist_view_update_thread(THREADLIST_VIEW(threadlist_view),
+				thread, info->num_shown,
+				(MAX(thread->number_of_responses_on_server,
+				     thread->number_of_responses_read)
+				 - info->bookmark_response_number));
+
   if (info->scroller_id != 0)
     {
       g_source_remove(info->scroller_id);
@@ -1865,7 +2031,7 @@ background_write_response(WorkerThread *employee, gpointer job_args)
   response.mailto = args->mailto;
   response.content = args->message;
 
-  snprintf(message, 4096, _("Posting response to %s@%s board"),
+  snprintf(message, 4096, _("Posting response to %s@%s board."),
 	   args->thread->title, args->thread->board->name);
   message_id = gtk_statusbar_push(args->application->statusbar,
 				  writing_netstat_id, message);
@@ -2063,7 +2229,8 @@ write_response(OchushaApplication *application)
     return;
 
   if (thread->flags & (OCHUSHA_BBS_THREAD_DAT_DROPPED
-		       | OCHUSHA_BBS_THREAD_STOPPED))
+		       | OCHUSHA_BBS_THREAD_STOPPED
+		       | OCHUSHA_BBS_THREAD_KAKO))
     return;
 
   if (!ochusha_bbs_thread_is_post_supported(thread))
@@ -2255,21 +2422,21 @@ start_thread_search_button_cb(GtkWidget *widget,
 
 
 void
-jump_to_new_comer_response_of_current_thread(OchushaApplication *application)
+jump_to_bookmark_of_current_thread(OchushaApplication *application)
 {
   GtkWidget *view;
   OchushaBBSThread *thread;
   get_current_thread(application, &view, &thread, NULL);
   if (view == NULL || thread == NULL)
     return;
-  jump_to_new_comer_response(view, thread);
+  jump_to_bookmark(view, thread);
 }
 
 
 static void
-jump_to_new_comer_button_cb(GtkWidget *widget, OchushaApplication *application)
+jump_to_bookmark_button_cb(GtkWidget *widget, OchushaApplication *application)
 {
-  jump_to_new_comer_response_of_current_thread(application);
+  jump_to_bookmark_of_current_thread(application);
 }
 
 
