@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.49 2003/12/16 19:47:43 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.52 2003/12/19 03:14:25 fuyu Exp $
  */
 
 #include "config.h"
@@ -461,6 +461,7 @@ ochusha_init_application(OchushaApplication *application)
   prepare_board_ui_initialization(application);
   prepare_thread_ui_initialization(application);
 
+  application->statusbar = GTK_STATUSBAR(gtk_statusbar_new());
 
   /* İν */
   initialize_boardlist(application);
@@ -473,8 +474,6 @@ ochusha_init_application(OchushaApplication *application)
 		   &application->contents_window);
 #endif
 
-  application->statusbar = GTK_STATUSBAR(gtk_statusbar_new());
-
   initialize_board_ui(application);
   initialize_thread_ui(application);
 
@@ -487,6 +486,7 @@ ochusha_init_application(OchushaApplication *application)
 #endif
 
   application->clipboard = NULL;
+  application->clipboard_primary = NULL;
 
 #if ENABLE_DEBUG_CONSOLE
   log_buffer = gtk_text_buffer_new(NULL);
@@ -515,11 +515,16 @@ ochusha_clipboard_set_text(OchushaApplication *application, const gchar *text)
   MAIN_UI_LOCK
   {
     if (application->clipboard == NULL)
-      application->clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+      {
+	application->clipboard = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+	application->clipboard_primary
+	  = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+      }
 #if DEBUG_GUI_MOST
     fprintf(stderr, "setting text on clipboard: \"%s\"\n", text);
 #endif
     gtk_clipboard_set_text(application->clipboard, text, -1);
+    gtk_clipboard_set_text(application->clipboard_primary, text, -1);
   }
   MAIN_UI_UNLOCK
 }
@@ -536,13 +541,14 @@ ochusha_open_url(OchushaApplication *application, const gchar *url,
   char *thread_id = NULL;
   unsigned int from = 0;
   unsigned int to = 0;
+  gboolean is_kako_html = FALSE;
   char *scheme;
   char command_line[COMMAND_LINE_BUFFER_SIZE];
 
   if (!use_web_browser
       && ochusha_bbs_table_check_url(application->table, url,
 				     &board, &board_url, &bbs_type,
-				     &thread_id, &from, &to))
+				     &thread_id, &from, &to, &is_kako_html))
     {
 #if 0
       if (board == NULL)
@@ -552,16 +558,20 @@ ochusha_open_url(OchushaApplication *application, const gchar *url,
 	  fprintf(stderr, "  url: %s\n", url);
 	  fprintf(stderr, "  board_url: %s\n", board_url);
 	  fprintf(stderr, "  bbs_type: %d\n", bbs_type);
-	  if (thread_id != NULL)
-	    fprintf(stderr, "  thread_id: %s\n", thread_id);
+	  fprintf(stderr, "  thread_id: %s\n", thread_id);
 	  fprintf(stderr, "  from=%d, to=%d\n", from, to);
+	  fprintf(stderr, "  is_kako_html=%d\n", is_kako_html);
 	}
 #endif
 
       if (board != NULL)
 	{
 	  /* ɽ */
-	  if (thread_id != NULL)
+	  if (is_kako_html)
+	    {
+	      ochusha_open_kako_thread(application, board, url, in_tab);
+	    }
+	  else if (thread_id != NULL)
 	    {
 	      ochusha_open_thread(application, board, thread_id, from, in_tab);
 	    }
@@ -980,6 +990,7 @@ static void
 write_optional_prefs(FILE *file, gpointer user_data)
 {
   OchushaApplication *application = (OchushaApplication *)user_data;
+  gchar *bookmark_text;
 
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, width);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, height);
@@ -1010,6 +1021,27 @@ write_optional_prefs(FILE *file, gpointer user_data)
 				  enable_transparent_a_bone);
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, disable_popup_a_bone);
 
+  OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, stick_bookmark);
+
+  if (application->bookmark_text != NULL
+      && *application->bookmark_text != '\0')
+    {
+      bookmark_text
+	= ochusha_utils_url_encode_string(application->bookmark_text);
+
+      if (bookmark_text != NULL && *bookmark_text != '\0')
+	{
+	  gchar *text = g_markup_escape_text(bookmark_text, -1);
+	  fprintf(file,
+		  "    <attribute name=\"bookmark_text\">\n"
+		  "      <string>%s</string>\n"
+		  "    </attribute>\n", text);
+	  g_free(text);
+	}
+      if (bookmark_text != NULL)
+	G_FREE(bookmark_text);
+    }
+
   OUTPUT_CONFIG_ATTRIBUTE_BOOLEAN(file, application, default_open_in_tab);
   OUTPUT_CONFIG_ATTRIBUTE_INT(file, application, show_mailto_mode);
 
@@ -1108,6 +1140,9 @@ configure_application(OchushaApplication *application)
       application->enable_transparent_a_bone = FALSE;
       application->disable_popup_a_bone = FALSE;
 
+      application->stick_bookmark = FALSE;
+      application->bookmark_text = G_STRDUP(_("==== My bookmark: I have read above. ===="));
+
       application->default_open_in_tab = FALSE;
       application->show_mailto_mode = OCHUSHA_DEFAULT_SHOW_MAILTO_MODE;
       application->a_bone_by_name = FALSE;
@@ -1181,6 +1216,7 @@ static void
 read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
 {
   OchushaApplication *application = (OchushaApplication *)user_data;
+  gchar *bookmark_text;
 
   application->width = ochusha_utils_get_attribute_int(pref_attrs, "width");
 
@@ -1286,6 +1322,27 @@ read_optional_prefs(GHashTable *pref_attrs, gpointer user_data)
     = ochusha_utils_get_attribute_boolean(pref_attrs,
 					  "disable_popup_a_bone");
 
+  application->stick_bookmark
+    = ochusha_utils_get_attribute_boolean(pref_attrs, "stick_bookmark");
+  application->bookmark_text = NULL;
+  bookmark_text = ochusha_utils_get_attribute_string(pref_attrs,
+						     "bookmark_text");
+  if (bookmark_text != NULL)
+    {
+      application->bookmark_text = ochusha_utils_url_decode_string(bookmark_text);
+      G_FREE(bookmark_text);
+    }
+
+  if (application->bookmark_text != NULL
+      && *application->bookmark_text == '\0')
+    {
+      G_FREE(application->bookmark_text);
+      application->bookmark_text = NULL;
+    }
+
+  if (application->bookmark_text == NULL)
+    application->bookmark_text = G_STRDUP(_("==== My bookmark: I have read above. ===="));
+
   application->show_mailto_mode
     = (ThreadViewMailtoMode)ochusha_utils_get_attribute_int(pref_attrs,
 							   "show_mailto_mode");
@@ -1857,6 +1914,38 @@ preference_dialog_response_cb(GtkWidget *widget, gint response_id,
 	      }
 	  }
 
+	tmp_data = g_object_get_data(G_OBJECT(widget), "stick_bookmark");
+	if (tmp_data != NULL)
+	  {
+	    application->stick_bookmark
+	      = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(tmp_data));
+	  }
+
+	tmp_data = g_object_get_data(G_OBJECT(widget), "bookmark_text");
+	if (tmp_data != NULL)
+	  {
+	    GtkTextBuffer *text_buffer = GTK_TEXT_BUFFER(tmp_data);
+	    GtkTextIter start_iter;
+	    GtkTextIter end_iter;
+	    gchar *new_text;
+	    gtk_text_buffer_get_start_iter(text_buffer, &start_iter);
+	    gtk_text_buffer_get_end_iter(text_buffer, &end_iter);
+	    new_text = gtk_text_buffer_get_text(text_buffer, &start_iter,
+						&end_iter, FALSE);
+	    if (*new_text == '\0')
+	      {
+		G_FREE(new_text);	/* ǥեȤΤ᤹ */
+		new_text = G_STRDUP(_("==== My bookmark: I have read above. ===="));
+		if (response_id == GTK_RESPONSE_APPLY)
+		  gtk_text_buffer_insert(text_buffer, &start_iter,
+					 new_text, -1);
+	      }
+
+	    if (application->bookmark_text != NULL)
+	      G_FREE(application->bookmark_text);
+	    application->bookmark_text = new_text;
+	  }
+
 	break;
       }
 
@@ -2300,6 +2389,65 @@ build_thread_view_setting_frame(OchushaApplication *application,
 }
 
 
+static GtkWidget *
+build_bookmark_setting_frame(OchushaApplication *application,
+			     GtkWidget *dialog)
+{
+  GtkWidget *outer_frame = gtk_frame_new(_("Setting of My Bookmark"));
+  GtkWidget *outer_vbox = gtk_vbox_new(FALSE, 0);
+  GtkWidget *tmp_widget;
+  GtkWidget *tmp_container;
+  GtkTextBuffer *text_buffer;
+  GtkTextIter iter;
+
+  gtk_frame_set_shadow_type(GTK_FRAME(outer_frame), GTK_SHADOW_ETCHED_OUT);
+  gtk_widget_show(outer_vbox);
+  gtk_container_add(GTK_CONTAINER(outer_frame), outer_vbox);
+
+
+  tmp_widget = gtk_radio_button_new_with_label(NULL, _("Do NOT move bookmark automatically."));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
+  g_object_set_data(G_OBJECT(dialog), "stick_bookmark", tmp_widget);
+
+  tmp_widget
+    = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(tmp_widget),
+						  _("Move bookmark to lead unread responses before loading and refreshment of thread."));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_widget, FALSE, FALSE, 0);
+
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp_widget),
+			       !application->stick_bookmark);
+
+
+  tmp_container = gtk_hbox_new(FALSE, 0);
+  tmp_widget = gtk_label_new(_("My Bookmark"));
+  gtk_widget_show(tmp_widget);
+  gtk_box_pack_start(GTK_BOX(tmp_container), tmp_widget, FALSE, FALSE, 5);
+  gtk_widget_show(tmp_container);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_container, FALSE, FALSE, 5);
+
+
+  tmp_container = gtk_scrolled_window_new(NULL, NULL);
+  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(tmp_container),
+				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(tmp_container),
+				      GTK_SHADOW_ETCHED_IN);
+  text_buffer = gtk_text_buffer_new(NULL);
+  gtk_text_buffer_get_start_iter(text_buffer, &iter);
+  gtk_text_buffer_insert(text_buffer, &iter, application->bookmark_text, -1);
+  g_object_set_data(G_OBJECT(dialog), "bookmark_text", text_buffer);
+
+  tmp_widget = gtk_text_view_new_with_buffer(text_buffer);
+  gtk_widget_show(tmp_widget);
+  gtk_container_add(GTK_CONTAINER(tmp_container), tmp_widget);
+  gtk_widget_show(tmp_container);
+  gtk_box_pack_start(GTK_BOX(outer_vbox), tmp_container, TRUE, TRUE, 0);
+
+  return outer_frame;
+}
+
+
 #if ENABLE_DEBUG_CONSOLE
 static void
 debug_console_response_cb(GtkWidget *widget, gint response_id,
@@ -2401,6 +2549,12 @@ open_preference_dialog(OchushaApplication *application)
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
 				 gtk_label_new(_("Setting of Thread View")));
 
+	tmp_widget = build_bookmark_setting_frame(application,
+						  preference_dialog);
+	gtk_widget_show(tmp_widget);
+	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), tmp_widget,
+				 gtk_label_new(_("Setting of My Bookmark")));
+
 	gtk_widget_show(preference_dialog);
       }
   }
