@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.c,v 1.30 2003/12/05 12:42:33 fuyu Exp $
+ * $Id: threadlist_view.c,v 1.32 2003/12/20 04:20:18 fuyu Exp $
  */
 
 #include "config.h"
@@ -64,19 +64,19 @@ static void threadlist_view_finalize(GObject *object);
 static void threadlist_view_destroy(GtkObject *object);
 
 static gboolean threadlist_view_title_not_equals(GtkTreeModel *model,
-						 gint column,
+						 int column,
 						 const gchar *key,
 						 GtkTreeIter *iter,
 						 gpointer search_data);
 
 static gboolean threadlist_view_move_cursor(GtkTreeView *tree_view,
 					    GtkMovementStep step,
-					    gint count);
+					    int count);
 
 static void threadlist_view_do_on_selection(ThreadlistView *view,
-					    gint what_to_do);
+					    int what_to_do);
 static void threadlist_view_do_on_thread_at_cursor(ThreadlistView *view,
-						   gint what_to_do);
+						   int what_to_do);
 
 static gboolean tree_view_motion_notify_event_cb(GtkWidget *widget,
 						 GdkEventMotion *event,
@@ -95,6 +95,11 @@ static void decorate_threadlist_entry(GtkTreeViewColumn *tree_column,
 				      GtkCellRenderer *renderer,
 				      GtkTreeModel *tree_model,
 				      GtkTreeIter *iter, gpointer view);
+static void text_set_num_of_responses_verbose(GtkTreeViewColumn *tree_column,
+					      GtkCellRenderer *renderer,
+					      GtkTreeModel *tree_model,
+					      GtkTreeIter *iter,
+					      gpointer view);
 static void text_set_cell_data_rank(GtkTreeViewColumn *tree_column,
 				    GtkCellRenderer *renderer,
 				    GtkTreeModel *tree_model,
@@ -200,7 +205,7 @@ struct _ThreadlistInfo
 
 
 static GtkTreeViewClass *parent_class = NULL;
-static gint threadlist_view_signals[LAST_SIGNAL] =
+static int threadlist_view_signals[LAST_SIGNAL] =
   { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GQuark list_entry_info_id;
@@ -374,7 +379,7 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 	NULL					/* type */
       },
     };
-  gint nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
+  int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
 
   popup_menu_item_factory = gtk_item_factory_new(GTK_TYPE_MENU,
 						 "<ThreadlistViewPopup>",
@@ -717,7 +722,9 @@ typedef enum
   THREADLIST_VISUAL_PARAMETER,
   THREADLIST_TITLE,
   THREADLIST_N_RESPONSES,
-  THREADLIST_N_RESPONSES_READ,
+  THREADLIST_N_RESPONSES_GOT,
+  THREADLIST_N_RESPONSES_UNGOT,
+  THREADLIST_N_RESPONSES_SHOWN,
   THREADLIST_N_RESPONSES_UNREAD,
   THREADLIST_RANK_DIFFERENCE,
   THREADLIST_LAST_MODIFIED,
@@ -828,7 +835,7 @@ ensure_threadlist_info(OchushaBBSThread *thread)
 
 
 static gboolean
-threadlist_view_title_not_equals(GtkTreeModel *model, gint column,
+threadlist_view_title_not_equals(GtkTreeModel *model, int column,
 				 const gchar *key, GtkTreeIter *iter,
 				 gpointer search_data)
 {
@@ -888,7 +895,7 @@ threadlist_view_title_not_equals(GtkTreeModel *model, gint column,
 
 static gboolean
 threadlist_view_move_cursor(GtkTreeView *tree_view, GtkMovementStep step,
-			    gint count)
+			    int count)
 {
   ThreadlistView *view = THREADLIST_VIEW(tree_view);
   gboolean result = FALSE;
@@ -1188,7 +1195,7 @@ threadlist_view_get_title_match_func(ThreadlistView *view)
 typedef struct _DoOnSelectionArgs
 {
   ThreadlistView *view;
-  gint what_to_do;
+  int what_to_do;
 } DoOnSelectionArgs;
 
 
@@ -1197,7 +1204,7 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
 		       GtkTreeIter *iter, DoOnSelectionArgs *args)
 {
   OchushaBBSThread *thread;
-  gint what_to_do = args->what_to_do;
+  int what_to_do = args->what_to_do;
 
   gtk_tree_model_get(model, iter,
 		     THREADLIST_DATA, &thread, -1);
@@ -1271,7 +1278,7 @@ do_on_selection_helper(GtkTreeModel *model, GtkTreePath *path,
 
 
 static void
-threadlist_view_do_on_selection(ThreadlistView *view, gint what_to_do)
+threadlist_view_do_on_selection(ThreadlistView *view, int what_to_do)
 {
   DoOnSelectionArgs args = { view, what_to_do };
   gtk_tree_selection_selected_foreach(view->selection,
@@ -1281,7 +1288,7 @@ threadlist_view_do_on_selection(ThreadlistView *view, gint what_to_do)
 
 
 static void
-threadlist_view_do_on_thread_at_cursor(ThreadlistView *view, gint what_to_do)
+threadlist_view_do_on_thread_at_cursor(ThreadlistView *view, int what_to_do)
 {
   GtkTreeIter iter;
   GtkTreePath *path = NULL;
@@ -1537,6 +1544,7 @@ tree_view_button_release_event_cb(GtkWidget *widget, GdkEventButton *event,
 
 #define RANK_UNSPECIFIED		-1
 #define RANK_DAT_DROPPED		-2
+#define RANK_KAKO_THREAD		-3
 
 #define RANK_NOT_CHANGED		0
 #define RANK_NEW_COMER_MAGIC		0x7fffffff
@@ -1545,17 +1553,18 @@ tree_view_button_release_event_cb(GtkWidget *widget, GdkEventButton *event,
 
 static void
 set_thread_values(ThreadlistView *view, GtkListStore *store,
+		  int num_shown, int num_unread,
 		  int last_rank, int new_rank, OchushaBBSThread *thread)
 {
   ThreadlistInfo *info = ensure_threadlist_info(thread);
-  int num_unread
+  int num_ungot
     = thread->number_of_responses_on_server - thread->number_of_responses_read;
-  num_unread = (num_unread >= 0 ? num_unread : 0);
+  num_ungot = (num_ungot >= 0 ? num_ungot : 0);
 
   if (new_rank != RANK_UNSPECIFIED)
     {
-      gint rank_diff;
-      if (new_rank == RANK_DAT_DROPPED)
+      int rank_diff;
+      if (new_rank == RANK_DAT_DROPPED || new_rank == RANK_KAKO_THREAD)
 	rank_diff = RANK_NOT_CHANGED;
       else
 	rank_diff = (last_rank == 0
@@ -1573,7 +1582,9 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
 		THREADLIST_VISUAL_PARAMETER, 0,
 		THREADLIST_TITLE, thread->title,
 		THREADLIST_N_RESPONSES, thread->number_of_responses_on_server,
-		THREADLIST_N_RESPONSES_READ, thread->number_of_responses_read,
+		THREADLIST_N_RESPONSES_GOT, thread->number_of_responses_read,
+		THREADLIST_N_RESPONSES_UNGOT, num_ungot,
+		THREADLIST_N_RESPONSES_SHOWN, num_shown,
 		THREADLIST_N_RESPONSES_UNREAD, num_unread,
 		THREADLIST_RANK_DIFFERENCE, rank_diff,
 		THREADLIST_LAST_MODIFIED, ochusha_bbs_thread_get_last_modified_utc(thread),
@@ -1585,7 +1596,9 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
       gtk_list_store_set(store, &info->iter,
 		THREADLIST_TITLE, thread->title,
 		THREADLIST_N_RESPONSES, thread->number_of_responses_on_server,
-		THREADLIST_N_RESPONSES_READ, thread->number_of_responses_read,
+		THREADLIST_N_RESPONSES_GOT, thread->number_of_responses_read,
+		THREADLIST_N_RESPONSES_UNGOT, num_ungot,
+		THREADLIST_N_RESPONSES_SHOWN, num_shown,
 		THREADLIST_N_RESPONSES_UNREAD, num_unread,
 		THREADLIST_LAST_MODIFIED, ochusha_bbs_thread_get_last_modified_utc(thread),
 		THREADLIST_DATA, thread,
@@ -1918,6 +1931,41 @@ decorate_threadlist_entry(GtkTreeViewColumn *tree_column,
 }
 
 
+static void
+text_set_num_of_responses_verbose(GtkTreeViewColumn *tree_column,
+				  GtkCellRenderer *renderer,
+				  GtkTreeModel *tree_model, GtkTreeIter *iter,
+				  gpointer view)
+{
+  GValue value = { 0, };
+  int num_responses;
+  int num_responses_got;
+  int num_responses_available;
+  gchar verbose_text[64];
+
+  g_return_if_fail(gtk_list_store_iter_is_valid(GTK_LIST_STORE(tree_model),
+						iter));
+
+  gtk_tree_model_get(tree_model, iter,
+		     THREADLIST_N_RESPONSES, &num_responses,
+		     THREADLIST_N_RESPONSES_GOT, &num_responses_got,
+		     -1);
+
+  num_responses_available = MAX(num_responses, num_responses_got);
+  if (num_responses_available == num_responses_got || num_responses_got == 0)
+    snprintf(verbose_text, 64, "%d", num_responses_available);
+  else
+    snprintf(verbose_text, 64, "%d (+%d)", num_responses_available,
+	     num_responses_available - num_responses_got);
+
+  g_value_init(&value, G_TYPE_STRING);
+  g_value_set_string(&value, verbose_text);
+
+  g_object_set_property((GObject *)renderer, "text", &value);
+  decorate_threadlist_entry(tree_column, renderer, tree_model, iter, view);
+}
+
+
 static void
 text_set_cell_data_rank(GtkTreeViewColumn *tree_column,
 			GtkCellRenderer *renderer,
@@ -1937,7 +1985,10 @@ text_set_cell_data_rank(GtkTreeViewColumn *tree_column,
   g_value_init(&value, G_TYPE_STRING);
   if (rank <= 0)
     {
-      g_value_set_static_string(&value, _("Dead"));
+      if (rank == RANK_KAKO_THREAD)
+	g_value_set_static_string(&value, _("Past"));
+      else
+	g_value_set_static_string(&value, _("Dead"));
     }
   else
     {
@@ -2150,10 +2201,14 @@ threadlist_view_append_number_of_responses(ThreadlistView *view)
 {
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
+  GValue xalign = { 0, };
 
   g_return_if_fail(IS_THREADLIST_VIEW(view) && view->n_res_column == NULL);
 
   renderer = gtk_cell_renderer_text_new();
+  g_value_init(&xalign, G_TYPE_FLOAT);
+  g_value_set_float(&xalign, 0.8);
+  g_object_set_property((GObject *)renderer, "xalign", &xalign);
   column = gtk_tree_view_column_new_with_attributes(_("#Res"),
 						    renderer, "text",
 						    THREADLIST_N_RESPONSES,
@@ -2169,23 +2224,81 @@ threadlist_view_append_number_of_responses(ThreadlistView *view)
 
 
 void
-threadlist_view_append_number_of_responses_read(ThreadlistView *view)
+threadlist_view_append_number_of_responses_got(ThreadlistView *view)
 {
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
+  GValue xalign = { 0, };
 
-  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->n_read_column == NULL);
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->n_got_column == NULL);
 
   renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("#Read"),
+  g_value_init(&xalign, G_TYPE_FLOAT);
+  g_value_set_float(&xalign, 0.8);
+  g_object_set_property((GObject *)renderer, "xalign", &xalign);
+  column = gtk_tree_view_column_new_with_attributes(_("#Got"),
 						renderer, "text",
-						THREADLIST_N_RESPONSES_READ,
+						THREADLIST_N_RESPONSES_GOT,
 						NULL);
   gtk_tree_view_column_set_cell_data_func(column, renderer,
 					  decorate_threadlist_entry,
 					  view, NULL);
   gtk_tree_view_append_column(&view->view, column);
-  view->n_read_column = column;
+  view->n_got_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
+void
+threadlist_view_append_number_of_responses_ungot(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+  GValue xalign = { 0, };
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->n_ungot_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  g_value_init(&xalign, G_TYPE_FLOAT);
+  g_value_set_float(&xalign, 0.8);
+  g_object_set_property((GObject *)renderer, "xalign", &xalign);
+  column = gtk_tree_view_column_new_with_attributes(_("#Ungot"),
+						renderer, "text",
+						THREADLIST_N_RESPONSES_UNGOT,
+						NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  decorate_threadlist_entry,
+					  view, NULL);
+  gtk_tree_view_append_column(&view->view, column);
+  view->n_ungot_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
+void
+threadlist_view_append_number_of_responses_shown(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+  GValue xalign = { 0, };
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->n_shown_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  g_value_init(&xalign, G_TYPE_FLOAT);
+  g_value_set_float(&xalign, 0.8);
+  g_object_set_property((GObject *)renderer, "xalign", &xalign);
+  column = gtk_tree_view_column_new_with_attributes(_("#Shown"),
+						renderer, "text",
+						THREADLIST_N_RESPONSES_SHOWN,
+						NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  decorate_threadlist_entry,
+					  view, NULL);
+  gtk_tree_view_append_column(&view->view, column);
+  view->n_shown_column = column;
 
   view->view_is_initialized = TRUE;
 }
@@ -2196,10 +2309,14 @@ threadlist_view_append_number_of_responses_unread(ThreadlistView *view)
 {
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
+  GValue xalign = { 0, };
 
   g_return_if_fail(IS_THREADLIST_VIEW(view) && view->n_unread_column == NULL);
 
   renderer = gtk_cell_renderer_text_new();
+  g_value_init(&xalign, G_TYPE_FLOAT);
+  g_value_set_float(&xalign, 0.8);
+  g_object_set_property((GObject *)renderer, "xalign", &xalign);
   column = gtk_tree_view_column_new_with_attributes(_("#Unread"),
 						renderer, "text",
 						THREADLIST_N_RESPONSES_UNREAD,
@@ -2214,6 +2331,34 @@ threadlist_view_append_number_of_responses_unread(ThreadlistView *view)
 }
 
 
+void
+threadlist_view_append_number_of_responses_verbose(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+  GValue xalign = { 0, };
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view)
+		   && view->n_res_verbose_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  g_value_init(&xalign, G_TYPE_FLOAT);
+  g_value_set_float(&xalign, 0.8);
+  g_object_set_property((GObject *)renderer, "xalign", &xalign);
+  column = gtk_tree_view_column_new_with_attributes(_("#Res"),
+						    renderer, "text",
+						    THREADLIST_N_RESPONSES,
+						    NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  text_set_num_of_responses_verbose,
+					  view, NULL);
+  gtk_tree_view_append_column(&view->view, column);
+  view->n_res_verbose_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
 void
 threadlist_view_append_rank_difference(ThreadlistView *view)
 {
@@ -2270,8 +2415,8 @@ threadlist_view_initialize_default_view(ThreadlistView *view)
   threadlist_view_append_mark(view);
   threadlist_view_append_title(view);
   threadlist_view_append_number_of_responses(view);
-  threadlist_view_append_number_of_responses_read(view);
-  threadlist_view_append_number_of_responses_unread(view);
+  threadlist_view_append_number_of_responses_got(view);
+  threadlist_view_append_number_of_responses_ungot(view);
   threadlist_view_append_rank_difference(view);
 }
 
@@ -2294,9 +2439,11 @@ threadlist_view_open(ThreadlistView *view)
 			     G_TYPE_INT,	/* rank */
 			     G_TYPE_INT,	/* flags */
 			     G_TYPE_STRING,	/* thread title */
-			     G_TYPE_INT,	/* number of responses read */
-			     G_TYPE_INT,	/* number of responses read */
-			     G_TYPE_INT,	/* number of responses unread*/
+			     G_TYPE_INT,	/* # of responses on server */
+			     G_TYPE_INT,	/* # of responses got */
+			     G_TYPE_INT,	/* # of responses ungot*/
+			     G_TYPE_INT,	/* # of responses shown */
+			     G_TYPE_INT,	/* # of responses unread */
 			     G_TYPE_INT,	/* rank difference */
 			     G_TYPE_LONG,	/* last modified */
 			     OCHUSHA_TYPE_BBS_THREAD);	/* data */
@@ -2333,9 +2480,11 @@ threadlist_view_close(ThreadlistView *view, OchushaBBSThread *last_thread)
 
 void
 threadlist_view_append_thread(ThreadlistView *view, OchushaBBSThread *thread,
-			      gint last_rank, gint new_rank)
+			      int num_shown, int num_unread,
+			      int last_rank, int new_rank)
 {
   ThreadlistInfo *info;
+  int rank;
   g_return_if_fail(IS_THREADLIST_VIEW(view));
   g_return_if_fail(OCHUSHA_IS_BBS_THREAD(thread));
 
@@ -2345,15 +2494,21 @@ threadlist_view_append_thread(ThreadlistView *view, OchushaBBSThread *thread,
 
   gtk_list_store_append(GTK_LIST_STORE(view->model), &info->iter);
   info->threadlist_view = view;
+
+  if (thread->flags & OCHUSHA_BBS_THREAD_KAKO)
+    rank = RANK_KAKO_THREAD;
+  else if (thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED)
+    rank = RANK_DAT_DROPPED;
+  else
+    rank = new_rank;
   set_thread_values(view, GTK_LIST_STORE(view->model),
-		    last_rank,
-		    ((thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED) == 0
-		     ? new_rank : RANK_DAT_DROPPED), thread);
+		    num_shown, num_unread, last_rank, rank, thread);
 }
 
 
 void
-threadlist_view_update_thread(ThreadlistView *view, OchushaBBSThread *thread)
+threadlist_view_update_thread(ThreadlistView *view, OchushaBBSThread *thread,
+			      int num_shown, int num_unread)
 {
   gpointer data = NULL;
   ThreadlistInfo *info;
@@ -2386,8 +2541,8 @@ threadlist_view_update_thread(ThreadlistView *view, OchushaBBSThread *thread)
 #endif
   g_return_if_fail(thread == data);
 
-  set_thread_values(view, GTK_LIST_STORE(view->model), 0, RANK_UNSPECIFIED,
-		    thread);
+  set_thread_values(view, GTK_LIST_STORE(view->model),
+		    num_shown, num_unread, 0, RANK_UNSPECIFIED, thread);
 }
 
 
