@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.11 2003/12/10 15:13:58 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.12 2003/12/19 03:11:47 fuyu Exp $
  */
 
 #include "config.h"
@@ -36,12 +36,24 @@
 #include <glib.h>
 
 #include <errno.h>
-#include <pthread.h>	/* GThread֤Ĥ */
+#include <fcntl.h>
+
+#include <pthread.h>
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <sys/types.h>
+#include <sys/mman.h>
+
+#include <zlib.h>
+
+
+#ifndef MAP_NOCORE
+# define MAP_NOCORE	0
+#endif
+
 
 #define OCHUSHA_INITIAL_ASYNC_BUFFER_SIZE	4096
 
@@ -347,6 +359,134 @@ ochusha_async_buffer_new(char *buffer, int length, DestructFunc *destructor)
 }
 
 
+OchushaAsyncBuffer *
+ochusha_async_buffer_new_with_file(int fd)
+{
+  BufferSyncObject *sync_object;
+  OchushaAsyncBuffer *buf
+    = OCHUSHA_ASYNC_BUFFER(g_object_new(OCHUSHA_TYPE_ASYNC_BUFFER, NULL));
+  gzFile gzfile;
+  int chunk_size;
+
+  LOCK_GLOBAL_LOCK
+    {
+      sync_object = get_sync_object();
+      ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list,
+						 buf);
+    }
+  UNLOCK_GLOBAL_LOCK
+
+  buf->fixed = FALSE;
+
+  buf->sync_object = sync_object;
+  buf->destructor = ochusha_async_buffer_free_when_finished;
+
+  buf->length = 0;
+  buf->buffer = 0;
+  buf->buffer_length = 0;
+
+  gzfile = gzdopen(fd, "rb");
+  if (gzfile == NULL)
+    return buf;
+
+  chunk_size = OCHUSHA_INITIAL_ASYNC_BUFFER_SIZE;
+
+  while (!gzeof(gzfile))
+    {
+      char *buffer;
+      int result;
+      if (!ochusha_async_buffer_ensure_free_space(buf, chunk_size))
+	abort();	/* why? */
+
+      buffer = (char *)buf->buffer + buf->length;
+      chunk_size = buf->buffer_length - buf->length;
+      result = gzread(gzfile, buffer, chunk_size);
+      if (result == -1)
+	{
+	  fprintf(stderr, "gzread cause some error\n");
+	  break;
+	}
+
+#if 0
+      fprintf(stderr, "0x%x bytes read\n", result);
+
+      if (result == 0)
+	fprintf(stderr, "EOF\n");
+      else if (result < chunk_size)
+	fprintf(stderr, "Close to EOF\n");
+#endif
+
+      buf->length += result;
+    }
+
+  gzclose(gzfile);
+
+#if DEBUG_MEMORY_USAGE
+  fprintf(stderr, "ochusha_async_buffer_new(): OchushaAsyncBuffer(%p)\n", buf);
+#endif
+
+  return buf;
+}
+
+
+static void
+munmap_when_finished(OchushaAsyncBuffer *buffer)
+{
+  munmap((void *)buffer->buffer, buffer->length);
+#if DEBUG_MMAP
+  fprintf(stderr, "mmapped buffer(at %p) is unmapped.\n",
+	  buffer->buffer);
+#endif
+}
+
+
+OchushaAsyncBuffer *
+ochusha_async_buffer_new_with_file_mmap(int fd)
+{
+  BufferSyncObject *sync_object;
+  char *buffer;
+  OchushaAsyncBuffer *buf
+    = OCHUSHA_ASYNC_BUFFER(g_object_new(OCHUSHA_TYPE_ASYNC_BUFFER, NULL));
+  off_t len;
+
+  LOCK_GLOBAL_LOCK
+  {
+    sync_object = get_sync_object();
+    ochusha_async_buffer_list = g_slist_append(ochusha_async_buffer_list,
+					       buf);
+  }
+  UNLOCK_GLOBAL_LOCK
+
+  len = lseek(fd, 0, SEEK_END);
+  buffer = mmap(NULL, len, PROT_READ, MAP_NOCORE | MAP_PRIVATE, fd, 0);
+  if (buffer == MAP_FAILED)
+    {
+      fprintf(stderr, "mmap failed due to: %s (%d)\n", strerror(errno), errno);
+      buf->buffer = NULL;
+      buf->length = 0;
+      buf->buffer_length = 0;
+      buf->destructor = NULL;
+    }
+  else
+    {
+      buf->buffer = buffer;
+      buf->length = len;
+      buf->buffer_length = len;
+      buf->destructor = munmap_when_finished;
+    }
+  close(fd);
+  
+  buf->fixed = FALSE;
+  buf->sync_object = sync_object;
+
+#if DEBUG_MEMORY_USAGE
+  fprintf(stderr, "ochusha_async_buffer_new_with_file_mmap(): OchushaAsyncBuffer(%p)\n", buf);
+#endif
+
+  return buf;
+}
+
+
 /* ƱߴϢ */
 /*
  * MEMO: OchushaAsyncBufferȤäƤƤΥåɤ⤷Υå
@@ -369,13 +509,12 @@ ochusha_async_buffer_new(char *buffer, int length, DestructFunc *destructor)
  *              ʤɤǧ뤳Ȥ롣
  */
 gboolean
-ochusha_async_buffer_active_ref(OchushaAsyncBuffer *buffer, const char *user)
+ochusha_async_buffer_active_ref(OchushaAsyncBuffer *buffer)
 {
   gboolean result;
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "OchushaAsyncBuffer(%p) is actively refered by %s\n",
-	  buffer, user);
+  fprintf(stderr, "OchushaAsyncBuffer(%p) is actively refered\n", buffer);
 #endif
   g_object_ref(G_OBJECT(buffer));
 
@@ -407,12 +546,12 @@ ochusha_async_buffer_active_ref(OchushaAsyncBuffer *buffer, const char *user)
 
 
 void
-ochusha_async_buffer_active_unref(OchushaAsyncBuffer *buffer, const char *user)
+ochusha_async_buffer_active_unref(OchushaAsyncBuffer *buffer)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr,
-	  "OchushaAsyncBuffer(%p) actively refered by %s is unrefered\n",
-	  buffer, user);
+	  "OchushaAsyncBuffer(%p) actively refered is unrefered\n",
+	  buffer);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -567,17 +706,16 @@ ochusha_async_buffer_terminate(OchushaAsyncBuffer *buffer)
  * ХåեTERMINATED֤ξFALSE֤
  */
 gboolean
-ochusha_async_buffer_update_length(OchushaAsyncBuffer *buffer, int length,
-				   const char *user)
+ochusha_async_buffer_update_length(OchushaAsyncBuffer *buffer, int length)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "\"%s\" called ochusha_async_buffer_update_length\n", user);
+  fprintf(stderr, "ochusha_async_buffer_update_length called\n");
 #endif
   LOCK_BUFFER_LOCK(buffer);
   buffer->length = length;
   UNLOCK_BUFFER_LOCK(buffer);
 
-  return ochusha_async_buffer_broadcast(buffer, user);
+  return ochusha_async_buffer_broadcast(buffer);
 }
 
 
@@ -585,17 +723,17 @@ ochusha_async_buffer_update_length(OchushaAsyncBuffer *buffer, int length,
  * ХåեTERMINATED֤ξFALSE֤
  */
 gboolean
-ochusha_async_buffer_fix(OchushaAsyncBuffer *buffer, const char *user)
+ochusha_async_buffer_fix(OchushaAsyncBuffer *buffer)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "\"%s\" called ochusha_async_buffer_fix\n", user);
+  fprintf(stderr, "ochusha_async_buffer_fix called\n");
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
   buffer->fixed = TRUE;
   UNLOCK_BUFFER_LOCK(buffer);
 
-  return ochusha_async_buffer_broadcast(buffer, user);
+  return ochusha_async_buffer_broadcast(buffer);
 }
 
 
@@ -603,11 +741,10 @@ ochusha_async_buffer_fix(OchushaAsyncBuffer *buffer, const char *user)
  * Out of memory⤷ϥХåեTERMINATED֤ξFALSE֤
  */
 gboolean
-ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, int length,
-			    const char *user)
+ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, int length)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "\"%s\" called ochusha_async_buffer_resize\n", user);
+  fprintf(stderr, "ochusha_async_buffer_resize called\n");
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -623,7 +760,7 @@ ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, int length,
 
   UNLOCK_BUFFER_LOCK(buffer);
 
-  return ochusha_async_buffer_broadcast(buffer, user);
+  return ochusha_async_buffer_broadcast(buffer);
 }
 
 
@@ -635,15 +772,14 @@ ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, int length,
  * ХåեTERMINATED֤ξFALSE֤
  */
 gboolean
-ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer, int length,
-				       const char *user)
+ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer, int length)
 {
   gboolean result = TRUE;
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr,
-	  "ochusha_async_buffer_ensure_free_space(%p, %d, \"%s\") called.\n",
-	  buffer, length, user);
+	  "ochusha_async_buffer_ensure_free_space(%p, %d) called.\n",
+	  buffer, length);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -658,8 +794,8 @@ ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer, int length,
     {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
       fprintf(stderr,
-	      "\"%s\" is suspended at ochusha_async_buffer_free_space.\n",
-	      user);
+	      "buffer(%p) is suspended at ochusha_async_buffer_free_space.\n",
+	      buffer);
 #endif
       buffer->number_of_suspended_users++;
 
@@ -692,16 +828,16 @@ ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer, int length,
 
 
 gboolean
-ochusha_async_buffer_append_data(OchushaAsyncBuffer *buffer, const char *data, int length,
-			 const char *user)
+ochusha_async_buffer_append_data(OchushaAsyncBuffer *buffer, const char *data,
+				 int length)
 {
   gboolean result = TRUE;
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "\"%s\" called ochusha_async_buffer_append_data\n", user);
+  fprintf(stderr, "ochusha_async_buffer_append_data called.\n");
 #endif
 
-  if (ochusha_async_buffer_ensure_free_space(buffer, length, user))
+  if (ochusha_async_buffer_ensure_free_space(buffer, length))
     {
       memcpy((char *)buffer->buffer + buffer->length, data, length);
       buffer->length += length;
@@ -709,7 +845,53 @@ ochusha_async_buffer_append_data(OchushaAsyncBuffer *buffer, const char *data, i
   else
     result = FALSE;
 
-  return ochusha_async_buffer_broadcast(buffer, user) && result;
+  return ochusha_async_buffer_broadcast(buffer) && result;
+}
+
+
+gboolean
+ochusha_async_buffer_read_file(OchushaAsyncBuffer *buffer, int fd)
+{
+  gzFile gzfile;
+  int chunk_size;
+  int result;
+
+  g_return_val_if_fail(fd >= 0, FALSE);
+
+  gzfile = gzdopen(fd, "rb");
+  g_return_val_if_fail(gzfile != NULL, FALSE);
+
+  chunk_size = OCHUSHA_INITIAL_ASYNC_BUFFER_SIZE;
+  result = 0;
+
+  while (!gzeof(gzfile))
+    {
+      char *buf;
+      if (!ochusha_async_buffer_ensure_free_space(buffer, chunk_size))
+	{
+	  result = -1;
+	  break;
+	}
+
+      buf = (char *)buffer->buffer + buffer->length;
+      chunk_size = buffer->buffer_length - buffer->length;
+      result = gzread(gzfile, buf, chunk_size);
+      if (result == -1)
+	{
+	  fprintf(stderr, "gzread cause some error\n");
+	  break;
+	}
+
+      if (!ochusha_async_buffer_update_length(buffer, buffer->length + result))
+	{
+	  result = -1;
+	  break;
+	}
+    }
+
+  gzclose(gzfile);
+
+  return result >= 0;
 }
 
 
@@ -728,14 +910,14 @@ ochusha_async_buffer_unlock(OchushaAsyncBuffer *buffer)
 
 
 gboolean
-ochusha_async_buffer_check_active(OchushaAsyncBuffer *buffer, const char *user)
+ochusha_async_buffer_check_active(OchushaAsyncBuffer *buffer)
 {
   gboolean result;
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr,
-	  "ochusha_async_buffer_check_active(%p, \"%s\")\n",
-	  buffer, user);
+	  "ochusha_async_buffer_check_active(%p)\n",
+	  buffer);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -744,8 +926,8 @@ ochusha_async_buffer_check_active(OchushaAsyncBuffer *buffer, const char *user)
     {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
       fprintf(stderr,
-	      "\"%s\" is suspended at ochusha_async_buffer_check_signal.\n",
-	      user);
+	      "buffer(%p) is suspended at ochusha_async_buffer_check_signal.\n",
+	      buffer);
 #endif
       buffer->number_of_suspended_users++;
 
@@ -757,7 +939,7 @@ ochusha_async_buffer_check_active(OchushaAsyncBuffer *buffer, const char *user)
 					 * ᥤ󥹥åɤΤΡ
 					 */
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr, "\"%s\" resumed.\n", user);
+      fprintf(stderr, "buffer(%p) resumed.\n", buffer);
 #endif
     }
   /* ХåեmutexunlockǤstateѤäƤǽΤ
@@ -775,13 +957,12 @@ ochusha_async_buffer_check_active(OchushaAsyncBuffer *buffer, const char *user)
 
 
 gboolean
-ochusha_async_buffer_signal(OchushaAsyncBuffer *buffer, const char *user)
+ochusha_async_buffer_signal(OchushaAsyncBuffer *buffer)
 {
   gboolean result;
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_signal(%p, \"%s\")\n",
-	  buffer, user);
+  fprintf(stderr, "ochusha_async_buffer_signal(%p)\n", buffer);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -790,7 +971,8 @@ ochusha_async_buffer_signal(OchushaAsyncBuffer *buffer, const char *user)
     {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
       fprintf(stderr,
-	      "\"%s\" is suspended at ochusha_async_buffer_signal.\n", user);
+	      "buffer(%p) is suspended at ochusha_async_buffer_signal.\n",
+	      buffer);
 #endif
       buffer->number_of_suspended_users++;
 
@@ -802,7 +984,7 @@ ochusha_async_buffer_signal(OchushaAsyncBuffer *buffer, const char *user)
 					 * ᥤ󥹥åɤΤΡ
 					 */
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr, "\"%s\" resumed.\n", user);
+      fprintf(stderr, "buffer(%p) resumed.\n", buffer);
 #endif
     }
   /* ХåեmutexunlockǤstateѤäƤǽΤ
@@ -824,13 +1006,12 @@ ochusha_async_buffer_signal(OchushaAsyncBuffer *buffer, const char *user)
 
 
 gboolean
-ochusha_async_buffer_broadcast(OchushaAsyncBuffer *buffer, const char *user)
+ochusha_async_buffer_broadcast(OchushaAsyncBuffer *buffer)
 {
   gboolean result;
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_broadcast(%p, \"%s\")\n",
-	  buffer, user);
+  fprintf(stderr, "ochusha_async_buffer_broadcast(%p)\n", buffer);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -839,8 +1020,8 @@ ochusha_async_buffer_broadcast(OchushaAsyncBuffer *buffer, const char *user)
     {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
       fprintf(stderr,
-	      "\"%s\" is suspended at ochusha_async_buffer_broadcast.\n",
-	      user);
+	      "buffer(%p) is suspended at ochusha_async_buffer_broadcast.\n",
+	      buffer);
 #endif
       buffer->number_of_suspended_users++;
 
@@ -852,7 +1033,7 @@ ochusha_async_buffer_broadcast(OchushaAsyncBuffer *buffer, const char *user)
 					 * ᥤ󥹥åɤΤΡ
 					 */
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr, "\"%s\" resumed.\n", user);
+      fprintf(stderr, "buffer(%p) resumed.\n", buffer);
 #endif
     }
   /* ХåեmutexunlockǤstateѤäƤǽΤ
@@ -872,19 +1053,18 @@ ochusha_async_buffer_broadcast(OchushaAsyncBuffer *buffer, const char *user)
 
 
 gboolean
-ochusha_async_buffer_wait(OchushaAsyncBuffer *buffer, const char *user)
+ochusha_async_buffer_wait(OchushaAsyncBuffer *buffer)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_wait(%p, \"%s\")\n",
-	  buffer, user);
+  fprintf(stderr, "ochusha_async_buffer_wait(%p)\n", buffer);
 #endif
 
   while (buffer->state == OCHUSHA_ASYNC_BUFFER_SUSPENDED)
     {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
       fprintf(stderr,
-	      "\"%s\" is suspended at ochusha_async_buffer_wait.\n",
-	      user);
+	      "buffer(%p) is suspended at ochusha_async_buffer_wait.\n",
+	      buffer);
 #endif
       buffer->number_of_suspended_users++;
 
@@ -897,7 +1077,7 @@ ochusha_async_buffer_wait(OchushaAsyncBuffer *buffer, const char *user)
        * ʥsuspend򤯤˥ᥤ󥹥åɤäƤΡ
        */
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-      fprintf(stderr, "\"%s\" resumed.\n", user);
+      fprintf(stderr, "buffer(%p) resumed.\n", buffer);
 #endif
     }
 
@@ -918,8 +1098,7 @@ ochusha_async_buffer_wait(OchushaAsyncBuffer *buffer, const char *user)
  * bufferΥåcontentionäƤȤˤTRUE֤
  */
 gboolean
-ochusha_async_buffer_is_busy(OchushaAsyncBuffer *buffer,
-			     const char *user)
+ochusha_async_buffer_is_busy(OchushaAsyncBuffer *buffer)
 {
   g_return_val_if_fail(OCHUSHA_IS_ASYNC_BUFFER(buffer), FALSE);
 
