@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.c,v 1.16 2003/12/01 07:26:37 fuyu Exp $
+ * $Id: ochusha_bbs_table.c,v 1.17 2003/12/19 03:13:49 fuyu Exp $
  */
 
 #include "config.h"
@@ -339,13 +339,15 @@ ochusha_bbs_table_check_url(OchushaBBSTable *table, const char *url,
 			    OchushaBulletinBoard **board_p,
 			    char **board_url_p, OchushaBBSType *bbs_type_p,
 			    char **thread_id_p,
-			    unsigned int *from_p, unsigned int *to_p)
+			    unsigned int *from_p, unsigned int *to_p,
+			    gboolean *is_kako_html_p)
 {
   char *board_url = NULL;
   gboolean result;
 
   if ((result = ochusha_utils_2ch_check_url(url, &board_url, bbs_type_p,
-					    NULL, thread_id_p, from_p, to_p)))
+					    NULL, thread_id_p, from_p, to_p,
+					    is_kako_html_p)))
     {
       if (board_p != NULL)
 	{
