@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.h,v 1.2 2003/11/19 20:07:07 fuyu Exp $
+ * $Id: ochusha_bbs_table.h,v 1.3 2003/12/19 03:13:49 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BBS_TABLE_H_
@@ -113,7 +113,8 @@ gboolean ochusha_bbs_table_check_url(OchushaBBSTable *table, const char *url,
 				     char **board_url_p,
 				     OchushaBBSType *bbs_type_p,
 				     char **thread_id_p,
-				     unsigned int *from_p, unsigned int *to_p);
+				     unsigned int *from_p, unsigned int *to_p,
+				     gboolean *is_kako_html_p);
 
 gboolean ochusha_bbs_table_read_boardlist_xml(OchushaBBSTable *table,
 					      OchushaConfig *conf);
