@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.h,v 1.11 2003/12/16 17:25:59 fuyu Exp $
+ * $Id: ochusha_bulletin_board.h,v 1.12 2003/12/19 03:13:49 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_BULLETIN_BOARD_H_
@@ -120,6 +120,9 @@ struct _OchushaBulletinBoardClass
 				  const char *id, const gchar *title);
   OchushaBBSThread *(*lookup_thread_by_url)(OchushaBulletinBoard *board,
 					    const char *url);
+  OchushaBBSThread *(*lookup_kako_thread_by_url)(OchushaBulletinBoard *board,
+						 OchushaNetworkBroker *broker,
+						 const char *url);
   OchushaAsyncBuffer *(*get_threadlist_source)(OchushaBulletinBoard *board,
 					OchushaNetworkBroker *broker,
 					OchushaNetworkBrokerCacheMode mode);
@@ -174,6 +177,10 @@ OchushaBBSThread *ochusha_bulletin_board_lookup_bbs_thread_by_id(
 OchushaBBSThread *ochusha_bulletin_board_lookup_bbs_thread_by_url(
 					OchushaBulletinBoard *board,
 					const char *url);
+OchushaBBSThread *ochusha_bulletin_board_lookup_kako_thread_by_url(
+					OchushaBulletinBoard *board,
+					OchushaNetworkBroker *broker,
+					const char *url);
 
 OchushaAsyncBuffer *ochusha_bulletin_board_get_threadlist_source(
 					OchushaBulletinBoard *board,
