@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_config.c,v 1.15 2003/12/16 17:25:59 fuyu Exp $
+ * $Id: ochusha_config.c,v 1.16 2003/12/19 21:13:47 fuyu Exp $
  */
 
 #include "config.h"
@@ -240,6 +240,57 @@ ochusha_config_cache_file_exist(const OchushaConfig *config, const char *url)
 }
 
 
+/*
+ *
+ * ꤵ줿URLФ륭åե뤬뤫ɤĴ٤롣
+ *
+ * å夬¸ߤȤˤϥåեΥѥ̾ɽʸ֤
+ * callerϥѥ̾ʸפˤʤäfree()̳餦
+ */
+char *
+ochusha_config_cache_find_file(const OchushaConfig *config, const char *url)
+{
+  char *cur_pos;
+  char pathname[PATH_MAX];
+  int stat_result;
+  struct stat sb;
+
+  if (url == NULL)
+    return NULL;
+
+  if (config->home == NULL)
+    return NULL;
+
+  if (url[strlen(url) - 1] == '/')
+    return NULL;	/* index.htmlʤɤξάϵʤ */
+
+  /*
+   * ݤʤΤurlhttp://server/.*ǤȷǤ
+   */
+  cur_pos = strstr(url, "http://");
+  if (cur_pos == NULL)
+    return NULL;
+
+  if (snprintf(pathname, PATH_MAX, "%s/cache/%s", config->home, url + 7)
+      >= PATH_MAX)
+    return NULL;
+
+  stat_result = stat(pathname, &sb);
+  if (stat_result == 0 && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+    return G_STRDUP(pathname);
+
+  if (snprintf(pathname, PATH_MAX, "%s/cache/%s.gz", config->home, url + 7)
+      >= PATH_MAX)
+    return NULL;
+
+  stat_result = stat(pathname, &sb);
+  if (stat_result == 0 && S_ISREG(sb.st_mode) && access(pathname, R_OK) == 0)
+    return G_STRDUP(pathname);
+
+  return NULL;
+}
+
+
 /*
  * ochusha_config_cache_open_file:
  *
