@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.41 2003/12/15 17:16:44 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.44 2003/12/20 04:16:47 fuyu Exp $
  */
 
 #include "config.h"
@@ -48,6 +48,7 @@
 #define DEBUG_DAT_MOST	0
 
 #define ANALYSIS_INTERVAL	100
+#define ENABLE_MODEST_PARSING	1
 
 
 static void ochusha_thread_2ch_class_init(OchushaThread2chClass *klass);
@@ -214,6 +215,12 @@ ochusha_thread_2ch_finalize(GObject *object)
       thread->base_path = NULL;
     }
 
+  if (thread->kako_html != NULL)
+    {
+      G_FREE(thread->kako_html);
+      thread->kako_html = NULL;
+    }
+
   if (thread->responses != NULL)
     {
       G_FREE(thread->responses);
@@ -237,11 +244,30 @@ ochusha_thread_2ch_finalize(GObject *object)
 }
 
 
+void
+ochusha_thread_2ch_set_kako_html_url(OchushaThread2ch *thread_2ch,
+				     const gchar *url)
+{
+  OchushaBBSThread *thread;
+  g_return_if_fail(OCHUSHA_IS_THREAD_2CH(thread_2ch));
+
+  if (thread_2ch->kako_html != NULL)
+    G_FREE(thread_2ch->kako_html);
+
+  thread_2ch->kako_html = url != NULL ? G_STRDUP(url) : NULL;
+
+  thread = OCHUSHA_BBS_THREAD(thread_2ch);
+  thread->flags |= OCHUSHA_BBS_THREAD_KAKO;
+}
+
+
 static void
 ochusha_thread_2ch_read_threadlist_element(OchushaBBSThread *thread,
 					   GHashTable *thread_attributes)
 {
   OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(thread);
+  thread_2ch->kako_html
+    = ochusha_utils_get_attribute_string(thread_attributes, "kako_html");
   thread_2ch->last_modified
     = ochusha_utils_get_attribute_string(thread_attributes, "last_modified");
 
@@ -273,7 +299,10 @@ ochusha_thread_2ch_write_threadlist_element(OchushaBBSThread *thread,
   if (parent_class->write_threadlist_element != NULL)
     (*parent_class->write_threadlist_element)(thread, threadlist_xml);
 
-  OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, thread_2ch, last_modified);
+  OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, thread_2ch, kako_html);
+  if (!(thread->flags
+	& (OCHUSHA_BBS_THREAD_DAT_DROPPED | OCHUSHA_BBS_THREAD_KAKO)))
+    OUTPUT_THREAD_ATTRIBUTE_STRING(threadlist_xml, thread_2ch, last_modified);
 }
 
 
@@ -324,7 +353,7 @@ ochusha_thread_2ch_check_url(OchushaBBSThread *thread, const char *url,
   gboolean result;
 
   if (!ochusha_utils_2ch_check_url(url, NULL, NULL, &board_id,
-				   &thread_id, from_p, to_p))
+				   &thread_id, from_p, to_p, NULL))
     return FALSE;
 
 #if 0
@@ -385,7 +414,6 @@ ochusha_thread_2ch_new(OchushaBoard2ch *board,
 const char *
 ochusha_thread_2ch_get_dat_url(OchushaThread2ch *thread_2ch)
 {
-  char url[PATH_MAX];
   OchushaBBSThread *thread;
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread_2ch), NULL);
 
@@ -398,11 +426,36 @@ ochusha_thread_2ch_get_dat_url(OchushaThread2ch *thread_2ch)
 		       && thread->id != NULL,
 		       NULL);
 
-  if (snprintf(url, PATH_MAX, "%sdat/%s.dat",
-	       thread->board->base_url, thread->id) >= PATH_MAX)
-    return NULL;
+  if (thread_2ch->kako_html == NULL)
+    {
+      char url[PATH_MAX];
+      if (snprintf(url, PATH_MAX, "%sdat/%s.dat",
+		   thread->board->base_url, thread->id) >= PATH_MAX)
+	return NULL;
+
+      thread_2ch->dat_url = G_STRDUP(url);
+    }
+  else
+    {
+      char *dat_url = G_STRNDUP(thread_2ch->kako_html,
+				strlen(thread_2ch->kako_html) + 2);
+      char *html_pos = strstr(dat_url, ".html");
+      if (html_pos != NULL)
+	{
+	  html_pos[1] = 'd';
+	  html_pos[2] = 'a';
+	  html_pos[3] = 't';
+	  html_pos[4] = '.';
+	  html_pos[5] = 'g';
+	  html_pos[6] = 'z';
+	  html_pos[7] = '\0';
+
+	  thread_2ch->dat_url = dat_url;
+	}
+      else
+	G_FREE(dat_url);
+    }
 
-  thread_2ch->dat_url = G_STRDUP(url);
   return thread_2ch->dat_url;
 }
 
@@ -418,7 +471,7 @@ ochusha_thread_2ch_get_base_url(OchushaBBSThread *thread)
   g_return_val_if_fail(OCHUSHA_IS_THREAD_2CH(thread), NULL);
 
   thread_2ch = OCHUSHA_THREAD_2CH(thread);
-  if (thread_2ch->base_url != NULL)
+  if (thread_2ch->base_url != NULL && thread_2ch->kako_html == NULL)
     return thread_2ch->base_url;
 
   base_path = ochusha_thread_2ch_get_base_path(thread_2ch);
@@ -431,6 +484,9 @@ ochusha_thread_2ch_get_base_url(OchushaBBSThread *thread)
     return NULL;
 
   thread_2ch->base_url = G_STRDUP(url);
+
+  if (thread_2ch->kako_html != NULL)
+    return thread_2ch->kako_html;
   return thread_2ch->base_url;
 }
 
@@ -475,9 +531,11 @@ ochusha_thread_2ch_get_responses_source(OchushaBBSThread *thread,
 
   thread_2ch = OCHUSHA_THREAD_2CH(thread);
 
-  if (thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED
-      || ((thread->flags & OCHUSHA_BBS_THREAD_STOPPED)
-	  && thread->number_of_responses_read > 0))
+  if (thread->flags & OCHUSHA_BBS_THREAD_KAKO)
+    mode = OCHUSHA_NETWORK_BROKER_CACHE_AS_IS;
+  else if (thread->flags & OCHUSHA_BBS_THREAD_DAT_DROPPED
+	   || ((thread->flags & OCHUSHA_BBS_THREAD_STOPPED)
+	       && thread->number_of_responses_read > 0))
     mode = OCHUSHA_NETWORK_BROKER_CACHE_ONLY;
 
   return ochusha_network_broker_read_from_url(broker,
@@ -658,6 +716,17 @@ extract_2ch_response(char *start_pos, char *eol_pos,
 }
 
 
+#if ENABLE_MODEST_PARSING
+static gboolean
+advance_parsing(gpointer data)
+{
+  ochusha_async_buffer_broadcast((OchushaAsyncBuffer *)data);
+  g_object_unref((GObject *)data);
+  return FALSE;
+}
+#endif
+
+
 /*
  * ochusha_thread_2ch_parse_responses
  *
@@ -710,7 +779,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
       status = &fake_status;
     }
 
-  if (!ochusha_async_buffer_active_ref(buffer, "ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses"))
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
 #if DEBUG_ASYNC_BUFFER_MOST
       fprintf(stderr, "buffer has been terminated.\n");
@@ -785,7 +854,11 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 #endif
 	while (rest_of_data > 0
 	       && (rest_of_responses > 0 || rest_of_responses == -1)
+#if ENABLE_MODEST_PARSING
+	       && interval-- > 0
+#else
 	       && (buffer_fixed || interval-- > 0)
+#endif
 	       && (eol_pos = memchr(cur_pos, '\n', rest_of_data)) != NULL)
 	  {
 	    OchushaBBSResponse response = { NULL, NULL, NULL, NULL };
@@ -905,7 +978,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	    continue;
 	  }
 
-	if (buffer_fixed)
+	if (buffer_fixed && eol_pos == NULL)
 	  break;
 
 	if (interval > 0)
@@ -920,11 +993,9 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	    if (no_wait)
 	      break;	/* ξ */
 
-	    if (ochusha_async_buffer_is_busy(buffer,
-		"ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses"))
+	    if (ochusha_async_buffer_is_busy(buffer))
 	      do {
-		if (!ochusha_async_buffer_wait(buffer,
-		"ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses"))
+		if (!ochusha_async_buffer_wait(buffer))
 		  {
 #if DEBUG_ASYNC_BUFFER_MOST
 		    fprintf(stderr, "ochusha_thread_2ch_parse_responses(): buffer has been terminated.\n");
@@ -943,20 +1014,36 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	  }
 	else
 	  {
+#if ENABLE_MODEST_PARSING
+	    if (ochusha_async_buffer_is_busy(buffer))
+	      {
+		/*
+		 * buffer˥륹åɤsuspend/terminate
+		 * ȿ뤿˥å
+		 */
+		if (!ochusha_async_buffer_wait(buffer))
+		  goto terminated;
+	      }
+	    else
+	      {
+		/*
+		 * GUIΥ쥹ݥ󥹤βܻؤ¸
+		 */
+		g_object_ref(buffer);
+		g_idle_add_full(G_PRIORITY_HIGH_IDLE + 15,
+				advance_parsing, buffer, NULL);
+		if (!ochusha_async_buffer_wait(buffer))
+		  goto terminated;
+	      }
+#else
 	    /*
 	     * buffer˥륹åɤsuspend/terminate
 	     * ȿ뤿˥å
 	     */
-	    if (ochusha_async_buffer_is_busy(buffer,
-		"ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses")
-		&& (!ochusha_async_buffer_wait(buffer,
-		"ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses")))
-	      {
-#if DEBUG_ASYNC_BUFFER_MOST
-		fprintf(stderr, "ochusha_thread_2ch_parse_responses(): buffer has been terminated.\n");
+	    if (ochusha_async_buffer_is_busy(buffer)
+		&& !ochusha_async_buffer_wait(buffer))
+	      goto terminated;
 #endif
-		break;
-	      }
 	  }
 	interval = ANALYSIS_INTERVAL;
 
@@ -968,6 +1055,8 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	    /* å夬äƤ뤳ȤϤ⤦狼ä */
 	    status->state
 	      = OCHUSHA_NETWORK_BROKER_BUFFER_STATE_DIRECT_ACCESS;
+	    memset(thread_2ch->responses, 0,
+		   thread_2ch->responses_length * sizeof(guint));
 	    goto retry_analysis;
 	  }
       }
@@ -978,21 +1067,25 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
   }
   ochusha_async_buffer_unlock(buffer);
 
-  if (status->last_modified != NULL)
+  if (!(thread->flags
+	& (OCHUSHA_BBS_THREAD_DAT_DROPPED | OCHUSHA_BBS_THREAD_KAKO)))
     {
-      if (thread_2ch->last_modified != NULL)
-	G_FREE(thread_2ch->last_modified);
-      thread_2ch->last_modified = G_STRDUP(status->last_modified);
+      if (status->last_modified != NULL)
+	{
+	  if (thread_2ch->last_modified != NULL)
+	    G_FREE(thread_2ch->last_modified);
+	  thread_2ch->last_modified = G_STRDUP(status->last_modified);
+	}
+
+      if (status->date != NULL)
+	{
+	  if (thread_2ch->date != NULL)
+	    G_FREE(thread_2ch->date);
+	  thread_2ch->date = G_STRDUP(status->date);
+	}
     }
 
-  if (status->date != NULL)
-    {
-      if (thread_2ch->date != NULL)
-	G_FREE(thread_2ch->date);
-      thread_2ch->date = G_STRDUP(status->date);
-    }
-
-  ochusha_async_buffer_active_unref(buffer, "ochusha_thread_2ch.c: ochusha_thread_2ch_parse_responses");
+  ochusha_async_buffer_active_unref(buffer);
 
   if (end_thread_cb != NULL)
     result = (*end_thread_cb)(thread, TRUE, callback_data);
