@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_jbbs.c,v 1.12 2003/12/14 19:30:00 fuyu Exp $
+ * $Id: ochusha_thread_jbbs.c,v 1.14 2003/12/20 04:19:36 fuyu Exp $
  */
 
 #include "config.h"
@@ -60,6 +60,7 @@
 #define DEBUG_DAT_MOST	0
 
 #define ANALYSIS_INTERVAL	100
+#define ENABLE_MODEST_PARSING	1
 
 
 static void ochusha_thread_jbbs_class_init(OchushaThreadJBBSClass *klass);
@@ -426,6 +427,17 @@ extract_jbbs_response(char *start_pos, char *eol_pos,
 }
 
 
+#if ENABLE_MODEST_PARSING
+static gboolean
+advance_parsing(gpointer data)
+{
+  ochusha_async_buffer_broadcast((OchushaAsyncBuffer *)data);
+  g_object_unref((GObject *)data);
+  return FALSE;
+}
+#endif
+
+
 /*
  * ochusha_thread_jbbs_parse_responses
  *
@@ -478,8 +490,7 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
       status = &fake_status;
     }
 
-  if (!ochusha_async_buffer_active_ref(buffer,
-		"ochusha_thread_jbbs.c: ochusha_thread_jbbs_parse_responses"))
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
 #if DEBUG_ASYNC_BUFFER_MOST
       fprintf(stderr, "buffer has been terminated.\n");
@@ -554,7 +565,11 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 #endif
 	while (rest_of_data > 0
 	       && (rest_of_responses > 0 || rest_of_responses == -1)
+#if ENABLE_MODEST_PARSING
+	       && interval-- > 0
+#else
 	       && (buffer_fixed || interval-- > 0)
+#endif
 	       && (eol_pos = memchr(cur_pos, '\n', rest_of_data)) != NULL)
 	  {
 	    OchushaBBSResponse response = { NULL, NULL, NULL, NULL };
@@ -661,11 +676,9 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 	    if (no_wait)
 	      break;	/* ξ */
 
-	    if (ochusha_async_buffer_is_busy(buffer,
-		"ochusha_thread_jbbs.c: ochusha_thread_jbbs_parse_responses"))
+	    if (ochusha_async_buffer_is_busy(buffer))
 	      do {
-		if (!ochusha_async_buffer_wait(buffer,
-		"ochusha_thread_jbbs.c: ochusha_thread_jbbs_parse_responses"))
+		if (!ochusha_async_buffer_wait(buffer))
 		  {
 #if DEBUG_ASYNC_BUFFER_MOST
 		    fprintf(stderr, "ochusha_thread_jbbs_parse_responses(): buffer has been terminated.\n");
@@ -676,20 +689,41 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
 	  }
 	else
 	  {
+#if ENABLE_MODEST_PARSING
+	    if (ochusha_async_buffer_is_busy(buffer))
+	      {
+		/*
+		 * buffer˥륹åɤsuspend/terminate
+		 * ȿ뤿˥å
+		 */
+		if (!ochusha_async_buffer_wait(buffer))
+		  goto terminated;
+	      }
+	    else
+	      {
+		/*
+		 * GUIΥ쥹ݥ󥹤βܻؤ¸
+		 */
+		g_object_ref(buffer);
+		g_idle_add_full(G_PRIORITY_HIGH_IDLE + 15,
+				advance_parsing, buffer, NULL);
+		if (!ochusha_async_buffer_wait(buffer))
+		  goto terminated;
+	      }
+#else
 	    /*
 	     * buffer˥륹åɤsuspend/terminate
 	     * ȿ뤿˥å
 	     */
-	    if (ochusha_async_buffer_is_busy(buffer,
-		"ochusha_thread_jbbs.c: ochusha_thread_jbbs_parse_responses")
-		&& (!ochusha_async_buffer_wait(buffer,
-		"ochusha_thread_jbbs.c: ochusha_thread_jbbs_parse_responses")))
+	    if (ochusha_async_buffer_is_busy(buffer)
+		&& !ochusha_async_buffer_wait(buffer))
 	      {
 #if DEBUG_ASYNC_BUFFER_MOST
 		fprintf(stderr, "ochusha_thread_jbbs_parse_responses(): buffer has been terminated.\n");
 #endif
 		break;
 	      }
+#endif
 	  }
 	interval = ANALYSIS_INTERVAL;
 
@@ -722,8 +756,7 @@ ochusha_thread_jbbs_parse_responses(OchushaBBSThread *thread,
       thread_2ch->date = G_STRDUP(status->date);
     }
 
-  ochusha_async_buffer_active_unref(buffer,
-		"ochusha_thread_jbbs.c: ochusha_thread_jbbs_parse_responses");
+  ochusha_async_buffer_active_unref(buffer);
 
   if (end_thread_cb != NULL)
     result = (*end_thread_cb)(thread, TRUE, callback_data);
@@ -1082,8 +1115,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
   fprintf(stderr, "convert_html_to_dat called\n");
 #endif
 
-  if (!ochusha_async_buffer_active_ref(html_buffer,
-				"ochusha_utils_jbbs.c: convert_html_to_dat()"))
+  if (!ochusha_async_buffer_active_ref(html_buffer))
     {
       g_signal_emit_by_name(G_OBJECT(args->broker),
 			    "access_terminated",
@@ -1092,15 +1124,13 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
       goto terminated;
     }
 
-  if (!ochusha_async_buffer_active_ref(dat_buffer,
-				"ochusha_utils_jbbs.c: convert_html_to_dat()"))
+  if (!ochusha_async_buffer_active_ref(dat_buffer))
     {
       g_signal_emit_by_name(G_OBJECT(args->broker),
 			    "access_terminated",
 			    dat_buffer,
 			    &signal_result);
-      ochusha_async_buffer_active_unref(html_buffer,
-				"ochusha_utils_jbbs.c: convert_html_to_dat()");
+      ochusha_async_buffer_active_unref(html_buffer);
       goto terminated;
     }
 
@@ -1221,8 +1251,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 		  fprintf(stderr, "breaking DAT for: \"%s\"\n",
 			  html_line_buffer);
 #endif
-		  ochusha_async_buffer_append_data(dat_buffer, "\n", 1,
-				"ochusha_utils_jbbs.c: convert_html_to_dat()");
+		  ochusha_async_buffer_append_data(dat_buffer, "\n", 1);
 		  response_number++;
 		}
 
@@ -1340,9 +1369,8 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 		  if (dat_line_buffer_rest > 0)
 		    {
 		      ochusha_async_buffer_append_data(dat_buffer,
-				dat_line_buffer,
-				dat_line_buffer_pos - dat_line_buffer,
-				"ochusha_utils_jbbs.c: convert_html_to_dat()");
+					dat_line_buffer,
+					dat_line_buffer_pos - dat_line_buffer);
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 		      {
 			dat_line_buffer_pos[0] = '\0';
@@ -1422,8 +1450,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 	  break;
 	}
 
-      if (!ochusha_async_buffer_wait(html_buffer,
-				"ochusha_utils_jbbs.c: convert_html_to_dat()"))
+      if (!ochusha_async_buffer_wait(html_buffer))
 	{
 #if DEBUG_HTML_TO_DAT_CONVERTER_MOST
 	  fprintf(stderr, "wait terminated.\n");
@@ -1446,10 +1473,8 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
     }
   ochusha_async_buffer_unlock(html_buffer);
 
-  ochusha_async_buffer_active_unref(html_buffer,
-				"ochusha_utils_jbbs.c: convert_html_to_dat()");
-  ochusha_async_buffer_active_unref(dat_buffer,
-				"ochusha_utils_jbbs.c: convert_html_to_dat()");
+  ochusha_async_buffer_active_unref(html_buffer);
+  ochusha_async_buffer_active_unref(dat_buffer);
 
   /* ɬפʤХåեƤ򥭥å˽񤭹 */
   if (dat_buffer->length > incoming_offset)
@@ -1477,8 +1502,7 @@ convert_html_to_dat(WorkerThread *employee, HTML2DATConverterJobArgs *args)
 #endif	      
     }
 
-  ochusha_async_buffer_fix(dat_buffer,
-			   "ochusha_utils_jbbs.c: convert_html_to_dat()");
+  ochusha_async_buffer_fix(dat_buffer);
 
  terminated:
   if (title_pattern != NULL)
@@ -1551,16 +1575,14 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 	  dat_buffer = ochusha_async_buffer_new(NULL, 0, NULL);
 	  incoming_offset = lseek(fd, 0, SEEK_END);
 	  lseek(fd, 0, SEEK_SET);
-	  if (ochusha_async_buffer_resize(dat_buffer, incoming_offset,
-					  "ochusha_utils_jbbs.c: ochusha_utils_jbbs_fake_responses_source()"))
+	  if (ochusha_async_buffer_resize(dat_buffer, incoming_offset))
 	    {
 	      char *tmp_pos;
 	      char *tail_pos;
 	      incoming_offset = read(fd, (void *)dat_buffer->buffer,
 				     incoming_offset);
 	      if (!ochusha_async_buffer_update_length(dat_buffer,
-						      incoming_offset,
-						      "ochusha_utils_jbbs.c: ochusha_utils_jbbs_fake_responses_source()"))
+						      incoming_offset))
 		{
 #if DEBUG_ASYNC_BUFFER
 		  fprintf(stderr, "ochusha_utils_jbbs_fake_responses_source(): buffer may have been terminated.\n");
@@ -1642,8 +1664,7 @@ ochusha_thread_jbbs_fake_responses_source(OchushaThread2ch *thread_2ch,
 				OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
 				_("Access failed: unknown reason"),
 				&signal_result);
-	  ochusha_async_buffer_fix(dat_buffer,
-				   "ochusha_utils_jbbs.c: ochusha_utils_jbbs_fake_responses_source()");
+	  ochusha_async_buffer_fix(dat_buffer);
 	}
       return dat_buffer;
     }
