@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_utils_2ch.c,v 1.25 2003/12/14 19:48:15 fuyu Exp $
+ * $Id: ochusha_utils_2ch.c,v 1.26 2003/12/19 03:13:38 fuyu Exp $
  */
 
 #include "config.h"
@@ -289,7 +289,8 @@ gboolean
 ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
 			    OchushaBBSType *bbs_type_p,
 			    char **board_id_p, char **thread_id_p,
-			    unsigned int *from_p, unsigned int *to_p)
+			    unsigned int *from_p, unsigned int *to_p,
+			    gboolean *is_kako_html_p)
 {
   char *server;
   char *abs_path;
@@ -581,11 +582,30 @@ ochusha_utils_2ch_check_url(const char *url, char **board_url_p,
       char *slash_pos = strchr(path_pos, '/');
       if (slash_pos != NULL)
 	{
+	  char *html_pos;
 	  slash_pos[0] = '\0';
 	  snprintf(board_url, PATH_MAX, "http://%s%s/", server, abs_path);
 	  board_id = G_STRDUP(abs_path + 1);
 	  slash_pos[0] = '/';
 	  parse_thread_path(slash_pos + 1, &thread_id, &from, &to);
+	  if (thread_id == NULL && is_kako_html_p != NULL
+	      && strncmp(slash_pos + 1, "kako/", 5) == 0
+	      && (html_pos = strstr(slash_pos + 6, ".html")) != NULL)
+	    {
+	      slash_pos = strrchr(slash_pos + 6, '/');
+	      if (slash_pos != NULL && slash_pos < html_pos)
+		{
+		  thread_id = G_STRNDUP(slash_pos + 1,
+					html_pos - slash_pos - 1);
+		  if (is_thread_id(thread_id))
+		    *is_kako_html_p = TRUE;
+		  else
+		    {
+		      G_FREE(thread_id);
+		      thread_id = NULL;
+		    }
+		}
+	    }
 	  bbs_type = OCHUSHA_BBS_TYPE_2CH;
 	}
 #if 0
@@ -857,7 +877,7 @@ ochusha_utils_2ch_analyze_bbsmenu(OchushaBBSTable *table,
   converter = iconv_open("UTF-8//IGNORE", "CP932");  /* ϡɥǥ */
   g_return_val_if_fail(converter != (iconv_t)-1, FALSE);
 
-  if (!ochusha_async_buffer_active_ref(buffer, "ochusha_utils_2ch.c: ochusha_utils_2ch_analyze_bbsmenu"))
+  if (!ochusha_async_buffer_active_ref(buffer))
     {
 #if DEBUG_ASYNC_BUFFER_MOST
       fprintf(stderr, "buffer has been terminated.\n");
@@ -948,7 +968,7 @@ ochusha_utils_2ch_analyze_bbsmenu(OchushaBBSTable *table,
 	if (buffer->fixed)
 	  break;
 
-	if (!ochusha_async_buffer_wait(buffer, "ochusha_utils_2ch.c: ochusha_utils_2ch_analyze_bbsmenu"))
+	if (!ochusha_async_buffer_wait(buffer))
 	  {
 	    /* bufferϤ޲٤callerˤޤ */
 #if DEBUG_ASYNC_BUFFER_MOST
@@ -968,7 +988,7 @@ ochusha_utils_2ch_analyze_bbsmenu(OchushaBBSTable *table,
   }
   ochusha_async_buffer_unlock(buffer);
 
-  ochusha_async_buffer_active_unref(buffer, "ochusha_utils_2ch.c: ochusha_utils_2ch_analyze_bbs_menu");
+  ochusha_async_buffer_active_unref(buffer);
 
   iconv_close(converter);
 
