@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.c,v 1.15 2003/11/30 16:43:56 fuyu Exp $
+ * $Id: utils.c,v 1.16 2003/12/18 14:07:43 fuyu Exp $
  */
 
 #define OCHUSHA_UTILS_IMPLEMENTATION
@@ -455,7 +455,13 @@ ochusha_utils_url_decode_string(const char *src)
   while (*cur_src != '\0')
     {
       if (*cur_src != '%')
-	*cur_dst++ = *cur_src++;
+	{
+	  if (*cur_src == '+')
+	    *cur_dst++ = ' ';
+	  else
+	    *cur_dst++ = *cur_src;
+	  cur_src++;
+	}
       else
 	{
 	  unsigned int code;
